@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -399,6 +407,7 @@ process_tag (const GstTagList *list, const gchar *tag, RBPlayerGst *player)
 		rb_debug ("Could not transform tag value type %s into %s",
 			  g_type_name (G_VALUE_TYPE (val)),
 			  g_type_name (G_VALUE_TYPE (newval)));
+		g_free (newval);
 		return;
 	}
 
