@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -35,8 +43,9 @@ G_BEGIN_DECLS
 #define RB_PLAYER_GST_DATA_TEE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), RB_TYPE_PLAYER_GST_DATA_TEE, RBPlayerGstDataTeeIface))
 
 typedef struct _RBPlayerGstDataTee RBPlayerGstDataTee;
+typedef struct _RBPlayerGstDataTeeIface RBPlayerGstDataTeeIface;
 
-typedef struct
+struct _RBPlayerGstDataTeeIface
 {
 	GTypeInterface	g_iface;
 
@@ -47,7 +56,7 @@ typedef struct
 	/* signals */
 	void		(*data_tee_inserted)	(RBPlayerGstDataTee *player, GstElement *data_tee);
 	void		(*data_tee_pre_remove)	(RBPlayerGstDataTee *player, GstElement *data_tee);
-} RBPlayerGstDataTeeIface;
+};
 
 GType		rb_player_gst_data_tee_get_type   (void);
 
