@@ -4,17 +4,17 @@
  *  Copyright (C) 2002 Jorn Baayen
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  it under the terms of the GNU Library General Public License as
+ *  published by the Free Software Foundation; either version 2, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
@@ -37,11 +37,7 @@ GdkPixbuf *eel_create_colorized_pixbuf (GdkPixbuf *src,
 
 char *     rb_utf_friendly_time        (time_t date);
 
-#ifdef HAVE_COLLATE_KEY_FILENAME
-#define rb_utf8_collate_key_for_filename g_utf8_collate_key_for_filename
-#else
-gchar*	rb_utf8_collate_key_for_filename (const gchar *str, gssize len);
-#endif
+char *     rb_make_valid_utf8 (const char *name, char substitute);
 
 G_END_DECLS
 
