@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -39,6 +47,8 @@ G_BEGIN_DECLS
 void rb_debug_init             (gboolean debug);
 void rb_debug_init_match       (const char *match);
 
+gboolean rb_debug_matches      (const char *func,
+				const char *file);
 void rb_debug_real             (const char *func,
 				const char *file,
 				int line,
