@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -340,7 +348,7 @@ get_mount_points (void)
 		mount_points = g_list_prepend (mount_points, uri);
 	}
 
-	g_list_foreach (volumes, (GFunc)gnome_vfs_volume_ref, NULL);
+	g_list_foreach (volumes, (GFunc)gnome_vfs_volume_unref, NULL);
 	g_list_free (volumes);
 
 	return mount_points;
@@ -368,6 +376,56 @@ rb_uri_get_mount_point (const char *uri)
 	return mount_point;
 }
 
+gchar *
+rb_uri_get_filesystem_type(const char *uri)
+{
+	GnomeVFSVolume *volume = NULL;
+	GnomeVFSVolumeMonitor *monitor = NULL;
+	gchar *mount_point_uri = NULL, *mount_point_path = NULL;
+	gchar *fstype = NULL;
+	GError *error = NULL;
+	
+	g_return_val_if_fail(uri != NULL, NULL);
+
+	monitor = gnome_vfs_get_volume_monitor ();
+	if (monitor == NULL) {
+		goto error;
+	}
+
+	mount_point_uri = rb_uri_get_mount_point(uri);
+	if (mount_point_uri == NULL) {
+		goto error;
+	}
+	
+	mount_point_path = g_filename_from_uri(mount_point_uri, NULL, &error);
+	if (error) {
+		g_warning("%s", error->message);
+		g_error_free(error);
+		goto error;
+	}
+
+	volume = gnome_vfs_volume_monitor_get_volume_for_path(monitor, mount_point_path);
+	if (volume  == NULL) {
+ 		goto error;
+	}
+	g_free(mount_point_path);
+	g_free(mount_point_uri);
+
+	fstype = gnome_vfs_volume_get_filesystem_type(volume);
+	
+	gnome_vfs_volume_unref(volume);
+	
+	return fstype;
+
+ error:
+	if (volume != NULL) {
+		gnome_vfs_volume_unref(volume);
+	}
+	g_free(mount_point_path);
+	g_free(mount_point_uri);
+	return NULL;
+}
+
 gboolean
 rb_uri_is_mounted (const char *uri)
 {
@@ -776,12 +834,25 @@ rb_string_list_contains (GList *list, const char *s)
 }
 
 void
-rb_list_deep_free (GList *list)
+rb_list_destroy_free (GList *list, GDestroyNotify destroyer)
 {
-	g_list_foreach (list, (GFunc)g_free, NULL);
+	g_list_foreach (list, (GFunc)destroyer, NULL);
 	g_list_free (list);
+
 }
 
+void
+rb_list_deep_free (GList *list)
+{
+	rb_list_destroy_free (list, (GDestroyNotify)g_free);
+}
+
+void
+rb_slist_deep_free (GSList *list)
+{
+	g_slist_foreach (list, (GFunc)g_free, NULL);
+	g_slist_free (list);
+}
 
 static void
 collate_keys_cb (gpointer key, gpointer value, GList **list)
@@ -937,3 +1008,77 @@ rb_str_in_strv (const char *needle, char **haystack)
 
 	return FALSE;
 }
+
+/**
+ * rb_set_tree_view_column_fixed_width:
+ * @treeview: the #GtkTreeView containing the column
+ * @column: the #GtkTreeViewColumn to size
+ * @renderer: the #GtkCellRenderer used in the column
+ * @strings: a NULL-terminated set of strings to base the size on
+ * @padding: a small amount of extra padding for the column
+ *
+ * Sets a fixed size for a tree view column based on
+ * a set of strings to be displayed in the column.
+ */
+void
+rb_set_tree_view_column_fixed_width (GtkWidget  *treeview,
+				     GtkTreeViewColumn *column,
+				     GtkCellRenderer *renderer,
+				     const char **strings,
+				     int padding)
+{
+	int max_width = 0;
+	int i = 0;
+
+	while (strings[i] != NULL) {
+		gint width;
+		g_object_set (renderer, "text", strings[i], NULL);
+		gtk_cell_renderer_get_size (renderer,
+					    GTK_WIDGET (treeview),
+					    NULL,
+					    NULL, NULL,
+					    &width, NULL);
+
+		if (width > max_width)
+			max_width = width;
+
+		i++;
+	}
+
+	gtk_tree_view_column_set_fixed_width (column, max_width + padding);
+}
+
+/**
+ * rb_scale_pixbuf_to_size:
+ * @pixbuf: the #GdkPixbuf containing the original image
+ * @size: a stock icon size
+ *
+ * Creates a new #GdkPixbuf from the original one, for a target of
+ * size, respecting the aspect ratio of the image.
+ */
+GdkPixbuf *
+rb_scale_pixbuf_to_size (GdkPixbuf *pixbuf, GtkIconSize size)
+{
+	int icon_size;
+	int width, height;
+	int d_width, d_height;
+
+	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
+
+	if (gtk_icon_size_lookup (size, &icon_size, NULL) == FALSE)
+		return NULL;
+
+	width = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+
+	if (width > height) {
+		d_width = icon_size;
+		d_height = d_width * height / width;
+	} else {
+		d_height = icon_size;
+		d_width = d_height * width / height;
+	}
+
+	return gdk_pixbuf_scale_simple (pixbuf, d_width, d_height, GDK_INTERP_BILINEAR);
+}
+
