@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -47,6 +55,8 @@ void rb_gtk_action_popup_menu (GtkUIManager *uimanager, const char *path);
 GtkWidget *rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size);
 
 gchar *rb_uri_get_mount_point (const char *uri);
+gchar *rb_uri_get_filesystem_type(const char *uri);
+
 gboolean rb_uri_is_mounted (const char *uri);
 
 
@@ -87,6 +97,9 @@ void rb_set_tree_view_column_fixed_width (GtkWidget *treeview,
 					  const char **strings,
 					  int padding);
 
+GdkPixbuf *rb_scale_pixbuf_to_size (GdkPixbuf *pixbuf,
+				    GtkIconSize size);
+
 G_END_DECLS
 
 #endif /* __RB_UTIL_H */
