@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -43,19 +51,21 @@
 #include "rb-file-helpers.h"
 #include "rb-plugin.h"
 #include "rb-removable-media-manager.h"
-#include "rb-static-playlist-source.h"
+#include "rb-ipod-static-playlist-source.h"
 #include "rb-util.h"
 #include "rhythmdb.h"
 #include "rb-cut-and-paste-code.h"
 
-static GObject *rb_ipod_source_constructor (GType type,
+#define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/ipod/paned_position"
+#define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/ipod/show_browser"
+
+static GObject *rb_ipod_source_constructor (GType type, 
 					    guint n_construct_properties,
 					    GObjectConstructParam *construct_properties);
 static void rb_ipod_source_dispose (GObject *object);
 
-static GObject *rb_ipod_source_constructor (GType type, guint n_construct_properties,
-			       GObjectConstructParam *construct_properties);
-static void rb_ipod_source_dispose (GObject *object);
+static char *impl_get_browser_key (RBSource *source);
+static char *impl_get_paned_key (RBBrowserSource *source);
 
 static gboolean impl_show_popup (RBSource *source);
 static void impl_move_to_trash (RBSource *asource);
@@ -71,6 +81,7 @@ static GList * impl_get_mime_types (RBRemovableMediaSource *source);
 static gboolean impl_track_added (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
 				  const char *dest,
+				  guint64 filesize,
 				  const char *mimetype);
 static char* impl_build_dest_uri (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
@@ -82,6 +93,11 @@ static gchar* ipod_get_filename_for_uri (const gchar *mount_point,
 					 const gchar *extension);
 static gchar* ipod_path_from_unix_path (const gchar *mount_point,
 					const gchar *unix_path);
+static gboolean rb_ipod_song_artwork_add_cb (RhythmDB *db,
+                                             RhythmDBEntry *entry,
+                                             const gchar *property_name,
+                                             const GValue *metadata,
+                                             RBiPodSource *isource);
 #endif
 static RhythmDB *get_db_for_source (RBiPodSource *source);
 
@@ -98,7 +114,7 @@ typedef struct
 	GHashTable *entry_map;
 
 	gboolean needs_shuffle_db;
-	RBStaticPlaylistSource *podcast_pl;
+	RBIpodStaticPlaylistSource *podcast_pl;
 
 	guint load_idle_id;
 
@@ -108,6 +124,13 @@ typedef struct
 	GQueue *offline_plays;
 } RBiPodSourcePrivate;
 
+#ifdef ENABLE_IPOD_WRITING
+typedef struct {
+	RBiPodSourcePrivate *priv;
+	GdkPixbuf *pixbuf;
+} RBiPodSongArtworkAddData;
+#endif
+
 RB_PLUGIN_DEFINE_TYPE(RBiPodSource,
 		      rb_ipod_source,
 		      RB_TYPE_REMOVABLE_MEDIA_SOURCE)
@@ -120,10 +143,13 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBRemovableMediaSourceClass *rms_class = RB_REMOVABLE_MEDIA_SOURCE_CLASS (klass);
+	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
 	object_class->constructor = rb_ipod_source_constructor;
 	object_class->dispose = rb_ipod_source_dispose;
 
+	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_get_browser_key  = impl_get_browser_key;
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_delete_thyself = impl_delete_thyself;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
@@ -141,6 +167,8 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 	rms_class->impl_track_added = NULL;
 #endif
 
+	browser_source_class->impl_get_paned_key = impl_get_paned_key;
+
 	g_type_class_add_private (klass, sizeof (RBiPodSourcePrivate));
 }
 
@@ -167,8 +195,6 @@ rb_ipod_source_name_changed_cb (RBiPodSource *source, GParamSpec *spec,
 static void
 rb_ipod_source_init (RBiPodSource *source)
 {	
-	g_signal_connect (G_OBJECT (source), "notify::name",
-			  (GCallback)rb_ipod_source_name_changed_cb, NULL);
 }
 
 static GObject *
@@ -180,12 +206,23 @@ rb_ipod_source_constructor (GType type, guint n_construct_properties,
 
 	source = RB_IPOD_SOURCE (G_OBJECT_CLASS (rb_ipod_source_parent_class)->
 			constructor (type, n_construct_properties, construct_properties));
+
 	songs = rb_source_get_entry_view (RB_SOURCE (source));
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
+        rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_FIRST_SEEN, FALSE);
 
 	rb_ipod_load_songs (source);
 
+#ifdef ENABLE_IPOD_WRITING                                      
+        RhythmDB *db = get_db_for_source (RB_IPOD_SOURCE (source));
+        g_signal_connect_object (db,
+                                 "entry-extra-metadata-notify::rb:coverArt",
+                                 G_CALLBACK (rb_ipod_song_artwork_add_cb),
+                                 RB_IPOD_SOURCE(source), 0);
+        g_object_unref (G_OBJECT (db));
+#endif
+
 	return G_OBJECT (source);
 }
 
@@ -261,6 +298,7 @@ rb_ipod_source_new (RBShell *shell,
 					       NULL));
 
 	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);
+        g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
 	return RB_REMOVABLE_MEDIA_SOURCE (source);
 }
@@ -301,16 +339,16 @@ playlist_track_removed (RhythmDBQueryModel *m,
 			RhythmDBEntry *entry,
 			gpointer data)
 {
-	RBStaticPlaylistSource *playlist = RB_STATIC_PLAYLIST_SOURCE (data);
-	Itdb_Playlist *ipod_pl = g_object_get_data (G_OBJECT (playlist), "itdb-playlist");
-	RBiPodSource *ipod = g_object_get_data (G_OBJECT (playlist), "ipod-source");
+	RBIpodStaticPlaylistSource *playlist = RB_IPOD_STATIC_PLAYLIST_SOURCE (data);
+	Itdb_Playlist *ipod_pl = rb_ipod_static_playlist_source_get_itdb_playlist (playlist);
+	RBiPodSource *ipod = rb_ipod_static_playlist_source_get_ipod_source (playlist);
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
 	Itdb_Track *track;
 
 	g_return_if_fail (ipod != NULL);
 	g_return_if_fail (ipod_pl != NULL);
 
-        track = g_hash_table_lookup (priv->entry_map, entry);
+	track = g_hash_table_lookup (priv->entry_map, entry);
 	g_return_if_fail (track != NULL);
 	rb_ipod_db_remove_from_playlist (priv->ipod_db, ipod_pl, track);
 }
@@ -319,9 +357,9 @@ static void
 playlist_track_added (GtkTreeModel *model, GtkTreePath *path,
 		      GtkTreeIter *iter, gpointer data)
 {
-	RBStaticPlaylistSource *playlist = RB_STATIC_PLAYLIST_SOURCE (data);
-	Itdb_Playlist *ipod_pl = g_object_get_data (G_OBJECT (playlist), "itdb-playlist");
-	RBiPodSource *ipod = g_object_get_data (G_OBJECT (playlist), "ipod-source");
+	RBIpodStaticPlaylistSource *playlist = RB_IPOD_STATIC_PLAYLIST_SOURCE (data);
+	Itdb_Playlist *ipod_pl = rb_ipod_static_playlist_source_get_itdb_playlist (playlist);
+	RBiPodSource *ipod = rb_ipod_static_playlist_source_get_ipod_source (playlist);
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
 	Itdb_Track *track;
 	RhythmDBEntry *entry;
@@ -340,7 +378,7 @@ static void
 add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 {
 	RBShell *shell;
-	RBSource *playlist_source;
+	RBIpodStaticPlaylistSource *playlist_source;
 	GList *it;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	RhythmDBEntryType entry_type;
@@ -351,10 +389,11 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 			  "entry-type", &entry_type,
 			  NULL);
 
-	playlist_source = rb_static_playlist_source_new (shell,
-							 playlist->name,
-							 FALSE,
-							 entry_type);
+	playlist_source = rb_ipod_static_playlist_source_new (shell,
+                                                              source,
+                                                              priv->ipod_db,
+                                                              playlist,
+                                                              entry_type);
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
 	for (it = playlist->members; it != NULL; it = it->next) {
@@ -370,7 +409,11 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 		g_free (filename);
 	}
 
-	g_object_ref (G_OBJECT(playlist_source));
+        /* RBSource derives from GtkWidget so its initial reference is
+         * floating. Since we need a ref for ourselves and we don't want it to
+         * be stolen by a GtkContainer, we sink the floating reference.
+         */
+	g_object_ref_sink (G_OBJECT(playlist_source));
 	playlist->userdata = playlist_source;
 	playlist->userdata_destroy = g_object_unref;
 	playlist->userdata_duplicate = g_object_ref;
@@ -383,13 +426,9 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 			          G_CALLBACK (playlist_track_removed),
 			          playlist_source);
 
-	g_object_set_data (G_OBJECT (playlist_source),
-			           "ipod-source", source);
-	g_object_set_data (G_OBJECT (playlist_source),
-			           "itdb-playlist", playlist);
 	if (itdb_playlist_is_podcasts(playlist))
-			priv->podcast_pl = RB_STATIC_PLAYLIST_SOURCE (playlist_source);
-	rb_shell_append_source (shell, playlist_source, RB_SOURCE (source));
+		priv->podcast_pl = playlist_source;
+	rb_shell_append_source (shell, RB_SOURCE (playlist_source), RB_SOURCE (source));
 	g_object_unref (shell);
 }
 
@@ -407,8 +446,7 @@ load_ipod_playlists (RBiPodSource *source)
 		playlist = (Itdb_Playlist *)it->data;
 		if (itdb_playlist_is_mpl (playlist)) {
 			continue;
-		}
-		if (playlist->is_spl) {
+		} else if (playlist->is_spl) {
 			continue;
 		}
 
@@ -433,6 +471,8 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	track->title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
 	track->album = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ALBUM);
 	track->artist = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ARTIST);
+	track->sort_artist = rhythmdb_entry_dup_string (entry,
+	                                                RHYTHMDB_PROP_ARTIST_SORTNAME);
 	track->genre = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_GENRE);
 	track->filetype = g_strdup (mimetype);
 	track->size = rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
@@ -441,7 +481,7 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	track->cd_nr = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
 	track->track_nr = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER);
 	track->bitrate = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE);
-	track->year = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DATE);
+	track->year = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR);
 	track->time_added = itdb_time_get_mac_time ();
 	track->time_played = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_LAST_PLAYED);
 	track->time_played = itdb_time_host_to_mac (track->time_played);
@@ -452,6 +492,8 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 
 	if (rhythmdb_entry_get_pointer (entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
 		track->mediatype = MEDIATYPE_PODCAST;
+		track->time_released = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_POST_TIME);
+		track->time_released = itdb_time_host_to_mac (track->time_released);
 	} else {
 		track->mediatype = MEDIATYPE_AUDIO;
 	}
@@ -608,6 +650,17 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 		g_value_unset (&value);
 	}
 
+	/* Set last added time */
+	if (song->time_added != 0) {
+		GValue value = {0, };
+		g_value_init (&value, G_TYPE_ULONG);
+		g_value_set_ulong (&value, itdb_time_mac_to_host (song->time_added));
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
+					       RHYTHMDB_PROP_FIRST_SEEN,
+					       &value);
+		g_value_unset (&value);
+	}
+
 	/* Set last played */
 	if (song->time_played != 0) {
 		GValue value = {0, };
@@ -627,6 +680,12 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	entry_set_string_prop (RHYTHMDB (db), entry,
 			       RHYTHMDB_PROP_ARTIST, song->artist);
 
+        if (song->sort_artist != NULL) {
+                entry_set_string_prop (RHYTHMDB (db), entry,
+                                       RHYTHMDB_PROP_ARTIST_SORTNAME,
+                                       song->sort_artist);
+        }
+
 	entry_set_string_prop (RHYTHMDB (db), entry,
 			       RHYTHMDB_PROP_ALBUM, song->album);
 
@@ -887,6 +946,9 @@ rb_ipod_load_songs (RBiPodSource *source)
 				      "name", name,
 				      NULL);
 		}
+                g_signal_connect (G_OBJECT (source), "notify::name",
+		  	          (GCallback)rb_ipod_source_name_changed_cb,
+                                  NULL);
 		priv->load_idle_id = g_idle_add ((GSourceFunc)load_ipod_db_idle_cb, source);
 	}
 	g_object_unref (volume);
@@ -1041,6 +1103,18 @@ impl_get_ui_actions (RBSource *source)
 	return actions;
 }
 
+static char *
+impl_get_browser_key (RBSource *source)
+{
+	return g_strdup (CONF_STATE_SHOW_BROWSER);
+}
+
+static char *
+impl_get_paned_key (RBBrowserSource *source)
+{
+	return g_strdup (CONF_STATE_PANED_POSITION);
+}
+
 static gboolean
 impl_show_popup (RBSource *source)
 {
@@ -1064,6 +1138,7 @@ impl_move_to_trash (RBSource *asource)
 	for (tem = sel; tem != NULL; tem = tem->next) {
 		RhythmDBEntry *entry;
 		const gchar *uri;
+		gchar *file;
 		Itdb_Track *track;
 
 		entry = (RhythmDBEntry *)tem->data;
@@ -1077,7 +1152,11 @@ impl_move_to_trash (RBSource *asource)
 
 		rb_ipod_db_remove_track (priv->ipod_db, track);
 		g_hash_table_remove (priv->entry_map, entry);
-		rhythmdb_entry_move_to_trash (db, entry);
+		file = g_filename_from_uri (uri, NULL, NULL);
+		if (file != NULL)
+			g_unlink (file);
+		g_free (file);
+		rhythmdb_entry_delete (db, entry);
 		rhythmdb_commit (db);
 	}
 
@@ -1135,6 +1214,92 @@ artwork_notify_cb (RhythmDB *db,
 	g_hash_table_remove (priv->artwork_request_map, entry);
 }
 
+static gboolean
+rb_add_artwork_whole_album_cb (GtkTreeModel *query_model,
+			       GtkTreePath *path,
+			       GtkTreeIter *iter,
+			       RBiPodSongArtworkAddData *artwork_data)
+{
+	RhythmDBEntry *entry;
+	Itdb_Track *song;	
+	
+	entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (query_model), iter);
+
+	song = g_hash_table_lookup (artwork_data->priv->entry_map, entry);
+	g_return_val_if_fail (song != NULL, FALSE);
+
+	if (song->has_artwork == 0x01) {		   
+		return FALSE;				
+	}
+	
+	rb_ipod_db_set_thumbnail (artwork_data->priv->ipod_db, song, artwork_data->pixbuf);
+	
+	return FALSE;	
+}			     
+
+static gboolean 
+rb_ipod_song_artwork_add_cb (RhythmDB *db,
+			     RhythmDBEntry *entry,
+			     const gchar *property_name,
+			     const GValue *metadata,
+			     RBiPodSource *isource)			     
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (isource);
+	Itdb_Device *device;
+	Itdb_Track *song;
+	GdkPixbuf *pixbuf;
+	GtkTreeModel *query_model;
+	RBiPodSongArtworkAddData artwork_data;
+        RhythmDBEntryType entry_type;
+
+	if (metadata == NULL) {
+		return FALSE;
+        }
+
+        if (G_VALUE_HOLDS (metadata, GDK_TYPE_PIXBUF) == FALSE) {
+		return FALSE;
+	}
+
+	song = g_hash_table_lookup (priv->entry_map, entry);		
+        if (song == NULL) {
+                return FALSE;
+        }
+
+	device = rb_ipod_db_get_device (priv->ipod_db);
+	if (device == NULL || itdb_device_supports_artwork (device) == FALSE) {
+		return FALSE;
+	}
+
+	if (song->album == NULL || song->artist == NULL) {
+		return FALSE;
+	}
+
+        g_object_get (G_OBJECT (isource), "entry-type", &entry_type, NULL);
+ 
+	pixbuf = GDK_PIXBUF (g_value_get_object (metadata));
+	
+	query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty (db));
+	
+	rhythmdb_do_full_query (db, RHYTHMDB_QUERY_RESULTS (query_model),
+                                RHYTHMDB_QUERY_PROP_EQUALS,
+                                RHYTHMDB_PROP_TYPE, entry_type,
+				RHYTHMDB_QUERY_PROP_EQUALS,
+				RHYTHMDB_PROP_ARTIST, song->artist,
+				RHYTHMDB_QUERY_PROP_EQUALS,
+				RHYTHMDB_PROP_ALBUM, song->album,
+				RHYTHMDB_QUERY_END);
+				
+	artwork_data.priv = priv;
+	artwork_data.pixbuf = pixbuf;
+				
+	gtk_tree_model_foreach (query_model,
+				(GtkTreeModelForeachFunc) rb_add_artwork_whole_album_cb,
+				&artwork_data);
+        g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);	
+	g_object_unref(query_model);
+	return FALSE;
+}
+												
 static void
 request_artwork (RBiPodSource *isource,
 		 RhythmDBEntry *entry,
@@ -1168,6 +1333,12 @@ add_to_podcasts (RBiPodSource *source, Itdb_Track *song)
 	gchar *filename;
 	const gchar *mount_path;
 
+        /* Set various flags indicating the Itdb_Track is a podcast */
+        song->skip_when_shuffling = 0x01;
+        song->remember_playback_position = 0x01;
+        song->mark_unplayed = 0x02;
+        song->flag4 = 0x03;
+
 	if (priv->podcast_pl == NULL) {
 		/* No Podcast playlist on the iPod, create a new one */
 		Itdb_Playlist *ipod_playlist;
@@ -1179,7 +1350,7 @@ add_to_podcasts (RBiPodSource *source, Itdb_Track *song)
 
 	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
   	filename = ipod_path_to_uri (mount_path, song->ipod_path);
- 	rb_static_playlist_source_add_location (priv->podcast_pl, filename, -1);
+ 	rb_static_playlist_source_add_location (RB_STATIC_PLAYLIST_SOURCE (priv->podcast_pl), filename, -1);
 	g_free (filename);
 }
 
@@ -1187,6 +1358,7 @@ static gboolean
 impl_track_added (RBRemovableMediaSource *source,
 		  RhythmDBEntry *entry,
 		  const char *dest,
+		  guint64 filesize,
 		  const char *mimetype)
 {
 	RBiPodSource *isource = RB_IPOD_SOURCE (source);
@@ -1200,6 +1372,7 @@ impl_track_added (RBRemovableMediaSource *source,
 		RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 		char *filename;
 		const char *mount_path;
+		Itdb_Device *device;
 
 		filename = g_filename_from_uri (dest, NULL, NULL);
 		mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
@@ -1210,18 +1383,10 @@ impl_track_added (RBRemovableMediaSource *source,
 		if (song->mediatype == MEDIATYPE_PODCAST) {
 			add_to_podcasts (isource, song);
 		}
-#ifdef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
-		/* reuse that #define since both functions were added to 
-		 * libgpod CVS HEAD around the same time
-		 */
-		Itdb_Device *device;
 		device = rb_ipod_db_get_device (priv->ipod_db);		
 		if (device && itdb_device_supports_artwork (device)) {
 			request_artwork (isource, entry, db, song);
 		}
-#else 
-		request_artwork (isource, entry, db, song);
-#endif
 		add_ipod_song_to_db (isource, db, song);
 		rb_ipod_db_add_track (priv->ipod_db, song);
 	}
@@ -1467,6 +1632,11 @@ impl_delete_thyself (RBSource *source)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	GList *p;
 
+        if (priv->ipod_db == NULL) {
+            RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
+            return;
+        }
+
 	for (p = rb_ipod_db_get_playlists (priv->ipod_db);
 	     p != NULL;
 	     p = p->next) {
@@ -1489,10 +1659,8 @@ impl_delete_thyself (RBSource *source)
 		}
 	}
 
-	if (priv->ipod_db != NULL) {
-		g_object_unref (G_OBJECT (priv->ipod_db));
-		priv->ipod_db = NULL;
-	}
+	g_object_unref (G_OBJECT (priv->ipod_db));
+	priv->ipod_db = NULL;
 
 	RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
 }
@@ -1512,3 +1680,27 @@ impl_get_mime_types (RBRemovableMediaSource *source)
 }
 
 #endif
+
+
+void
+rb_ipod_source_new_playlist (RBiPodSource *source)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	Itdb_Playlist *ipod_playlist;
+
+	ipod_playlist = itdb_playlist_new (_("New playlist"), FALSE);
+	rb_ipod_db_add_playlist (priv->ipod_db, ipod_playlist);
+	add_rb_playlist (source, ipod_playlist);
+}
+
+void
+rb_ipod_source_remove_playlist (RBiPodSource *ipod_source,
+				RBSource *source)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod_source);
+	RBIpodStaticPlaylistSource *playlist_source = RB_IPOD_STATIC_PLAYLIST_SOURCE (source);
+
+	rb_source_delete_thyself (source);
+	rb_ipod_db_remove_playlist (priv->ipod_db, rb_ipod_static_playlist_source_get_itdb_playlist (playlist_source));
+}
+
