@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -272,6 +280,7 @@ rb_iradio_source_constructor (GType type,
 {
 	RBIRadioSource *source;
 	RBShell *shell;
+	GtkAction *action;
 
 	source = RB_IRADIO_SOURCE (G_OBJECT_CLASS (rb_iradio_source_parent_class)
 			->constructor (type, n_construct_properties, construct_properties));
@@ -291,6 +300,14 @@ rb_iradio_source_constructor (GType type,
 								       G_N_ELEMENTS (rb_iradio_source_actions),
 								       source);
 
+	action = gtk_action_group_get_action (source->priv->action_group,
+                                              "MusicNewInternetRadioStation");
+        /* Translators: this is the toolbar button label for 
+           New Interner Radio Station action.
+           Don't include the prefix "radio|" in the translation. */
+        g_object_set (action, "short-label", Q_("radio|New"), NULL);
+
+
 	/* set up stations view */
 	source->priv->stations = rb_entry_view_new (source->priv->db, G_OBJECT (source->priv->player),
 						    CONF_STATE_IRADIO_SORTING,
@@ -850,27 +867,16 @@ rb_iradio_source_do_query (RBIRadioSource *source)
 	source->priv->setting_new_query = FALSE;
 }
 
-#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 static void
 handle_playlist_entry_cb (TotemPlParser *playlist,
 			  const char *uri,
 			  GHashTable *metadata,
 			  RBIRadioSource *source)
-#else
-static void
-handle_playlist_entry_cb (TotemPlParser *playlist,
-			  const char *uri,
-			  const char *title,
-			  const char *genre,
-			  RBIRadioSource *source)
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 {
-#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 	const char *title, *genre;
 
 	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
 	genre = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_GENRE);
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 	rb_iradio_source_add_station (source, uri, title, genre);
 }
 
@@ -885,19 +891,12 @@ rb_iradio_source_add_from_playlist (RBIRadioSource *source,
 	if (real_uri)
 		uri = real_uri;
 
-#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 	g_signal_connect_object (parser, "entry-parsed",
 				 G_CALLBACK (handle_playlist_entry_cb),
 				 source, 0);
-#else
-	g_signal_connect_object (parser, "entry",
-				 G_CALLBACK (handle_playlist_entry_cb),
-				 source, 0);
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
-	if (g_object_class_find_property (G_OBJECT_GET_CLASS (parser), "recurse"))
-		g_object_set (parser, "recurse", FALSE, NULL);
+	g_object_set (parser, "recurse", FALSE, NULL);
 
-	switch (totem_pl_parser_parse (parser, uri, TRUE)) {
+	switch (totem_pl_parser_parse (parser, uri, FALSE)) {
 	case TOTEM_PL_PARSER_RESULT_UNHANDLED:
 	case TOTEM_PL_PARSER_RESULT_IGNORED:
 		/* maybe it's the actual stream URL, then */
