@@ -8,6 +8,14 @@
 # the Free Software Foundation; either version 2, or (at your option)
 # any later version.
 #
+# The Rhythmbox authors hereby grants permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and Rhythmbox. This permission is above and beyond the permissions granted
+# by the GPL license by which Rhythmbox is covered. If you modify this code
+# you may extend this exception to your version of the code, but you are not
+# obligated to do so. If you do not wish to do so, delete this exception
+# statement from your version.
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -39,10 +47,16 @@ class TrackListHandler(xml.sax.handler.ContentHandler):
 	def endElement(self, name):
 		if name == "Track":
 			try:
+				# prefer ogg streams to mp3
+				if 'oggurl' in self.__track:
+					trackurl = self.__track['oggurl']
+				else:
+					trackurl = self.__track['url']
+	
 				# add the track to the source
-				entry = self.__db.entry_lookup_by_location (self.__track['url'])
+				entry = self.__db.entry_lookup_by_location (trackurl)
 				if entry == None:
-					entry = self.__db.entry_new(self.__entry_type, self.__track['url'])
+					entry = self.__db.entry_new(self.__entry_type, trackurl)
 
 				# if year is not set, use launch date instead
 				try:
@@ -61,7 +75,7 @@ class TrackListHandler(xml.sax.handler.ContentHandler):
 					duration = parse_int(self.__track['seconds'])
 				except ValueError:
 					duration = 0
-				
+
 				self.__db.set(entry, rhythmdb.PROP_ARTIST, self.__track['artist'])
 				self.__db.set(entry, rhythmdb.PROP_ALBUM, self.__track['albumname'])
 				self.__db.set(entry, rhythmdb.PROP_TITLE, self.__track['trackname'])
@@ -69,10 +83,13 @@ class TrackListHandler(xml.sax.handler.ContentHandler):
 				self.__db.set(entry, rhythmdb.PROP_DATE, date)
 				self.__db.set(entry, rhythmdb.PROP_GENRE, self.__track['mp3genre'])
 				self.__db.set(entry, rhythmdb.PROP_DURATION, duration)
-				self.__sku_dict[self.__track['url']] = self.__track['albumsku']
-				self.__home_dict[self.__track['url']] = self.__track['home']
-				self.__buy_dict[self.__track['url']] = self.__track['buy'].replace("buy_album", "buy_cd", 1)
-				self.__art_dict[self.__track['url']] = self.__track['cover_small']
+
+				key = str(trackurl)
+				sku = intern(str(self.__track['albumsku']))
+				self.__sku_dict[key] = sku
+				self.__home_dict[sku] = str(self.__track['home'])
+				self.__buy_dict[sku] = str(self.__track['buy'].replace("buy_album", "buy_cd", 1))
+				self.__art_dict[sku] = str(self.__track['cover_small'])
 
 				self.__db.commit()
 			except Exception,e: # This happens on duplicate uris being added
