@@ -9,6 +9,14 @@
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -255,7 +263,7 @@ rb_plugin_find_file (RBPlugin *plugin,
 		  ret, file, priv->name);
 
 	/* ensure it's an absolute path, so doesn't confuse rb_glade_new et al */
-	if (ret[0] != '/') {
+	if (ret != NULL && ret[0] != '/') {
 		char *pwd = g_get_current_dir ();
 		char *path = g_strconcat (pwd, G_DIR_SEPARATOR_S, ret, NULL);
 		g_free (ret);
