@@ -7,6 +7,14 @@
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
+ * The Rhythmbox authors hereby grants permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Rhythmbox. This permission is above and beyond the permissions granted
+ * by the GPL license by which Rhythmbox is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -103,7 +111,7 @@ static PyTypeObject *PyRBPlugin_Type;
 
 G_DEFINE_TYPE (RBPythonModule, rb_python_module, G_TYPE_TYPE_MODULE);
 
-static void
+void
 rb_python_module_init_python (void)
 {
 	PyObject *pygtk, *mdict, *require;
@@ -162,6 +170,10 @@ rb_python_module_init_python (void)
 	/* import gobject */
 	init_pygobject ();
 
+	/* disable pyg* log hooks, since ours is more interesting */
+#ifdef pyg_disable_warning_redirections
+	pyg_disable_warning_redirections ();
+#endif
 #ifndef PYGOBJECT_CAN_MARSHAL_GVALUE
 	pyg_register_gtype_custom (G_TYPE_VALUE, pyg_value_g_value_as_pyobject, pyg_value_g_value_from_pyobject);
 #endif
@@ -169,6 +181,8 @@ rb_python_module_init_python (void)
 	/* import gtk */
 	init_pygtk ();
 
+	pyg_enable_threads ();
+
 	gtk = PyImport_ImportModule ("gtk");
 	if (gtk == NULL) {
 		g_warning ("Could not import gtk");
@@ -318,6 +332,19 @@ rb_python_module_load (GTypeModule *gmodule)
 	return FALSE;
 }
 
+static gboolean
+rb_python_module_load_with_gil (GTypeModule *module)
+{
+	PyGILState_STATE state;
+	gboolean ret;
+
+	state = pyg_gil_state_ensure ();
+	ret = rb_python_module_load (module);
+	pyg_gil_state_release (state);
+
+	return ret;
+}
+
 static void
 rb_python_module_unload (GTypeModule *module)
 {
@@ -332,14 +359,18 @@ rb_python_module_new_object (RBPythonModule *module)
 {
 	RBPythonModulePrivate *priv = RB_PYTHON_MODULE_GET_PRIVATE (module);
 	RBPythonObject *object;
+	PyGILState_STATE state;
 
 	if (priv->type == 0)
 		return NULL;
 
+	state = pyg_gil_state_ensure ();
 	rb_debug ("Creating object of type %s", g_type_name (priv->type));
 	object = (RBPythonObject*) (g_object_new (priv->type,
 						  "name", priv->module,
 						  NULL));
+	pyg_gil_state_release (state);
+
 	if (object->instance == NULL) {
 		g_warning ("could not instantiate python object");
 		return NULL;
@@ -437,13 +468,8 @@ rb_python_module_class_init (RBPythonModuleClass *class)
 
 	g_type_class_add_private (object_class, sizeof (RBPythonModulePrivate));
 
-	module_class->load = rb_python_module_load;
+	module_class->load = rb_python_module_load_with_gil;
 	module_class->unload = rb_python_module_unload;
-
-	/* Init python subsystem, this should happen only once
-	 * in the process lifetime, and doing it here is ok since
-	 * class_init is called once */
-	rb_python_module_init_python ();
 }
 
 RBPythonModule *
@@ -470,6 +496,7 @@ rb_python_module_new (const gchar *path,
 
 /* --- these are not module methods, they are here out of convenience --- */
 
+#if 0
 static gint idle_garbage_collect_id = 0;
 
 static gboolean
@@ -482,15 +509,19 @@ run_gc (gpointer data)
 
 	return ret;
 }
+#endif
 
 void
 rb_python_garbage_collect ()
 {
+#if 0
 	if (Py_IsInitialized() && idle_garbage_collect_id == 0) {
 		idle_garbage_collect_id = g_idle_add (run_gc, NULL);
 	}
+#endif
 }
 
+#if 0
 static gboolean
 finalise_collect_cb (gpointer data)
 {
@@ -503,10 +534,12 @@ finalise_collect_cb (gpointer data)
 
 	return TRUE;
 }
+#endif
 
 void
 rb_python_shutdown ()
 {
+#if 0
 	if (Py_IsInitialized ()) {
 		if (idle_garbage_collect_id != 0) {
 			g_source_remove (idle_garbage_collect_id);
@@ -522,4 +555,5 @@ rb_python_shutdown ()
 		/* disable for now, due to bug 334188
 		Py_Finalize ();*/
 	}
+#endif
 }
