@@ -1,4 +1,4 @@
-/* 
+/*
  *  arch-tag: Header for RhythmDB property GtkTreeModel impl.
  *
  *  Copyright (C) 2003 Colin Walters <walters@rhythmbox.org>
@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -15,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -47,21 +55,26 @@ typedef enum
 	RHYTHMDB_PROPERTY_MODEL_COLUMN_LAST,
 } RhythmDBPropertyModelColumn;
 
+GType rhythmdb_property_model_column_get_type (void);
+#define RHYTHMDB_TYPE_PROPERTY_MODEL_COLUMN (rhythmdb_property_model_column_get_type ())
+
+typedef struct _RhythmDBPropertyModel RhythmDBPropertyModel;
+typedef struct _RhythmDBPropertyModelClass RhythmDBPropertyModelClass;
 typedef struct RhythmDBPropertyModelPrivate RhythmDBPropertyModelPrivate;
 
-typedef struct
+struct _RhythmDBPropertyModel
 {
 	GObject parent;
 
 	RhythmDBPropertyModelPrivate *priv;
-} RhythmDBPropertyModel;
+};
 
-typedef struct
+struct _RhythmDBPropertyModelClass
 {
 	GObjectClass parent;
 
-	void (*pre_row_deletion) (void);
-} RhythmDBPropertyModelClass;
+	void (*pre_row_deletion) (RhythmDBPropertyModel *model);
+};
 
 GType			rhythmdb_property_model_get_type	(void);
 
