@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -262,6 +270,7 @@ enum
 	ENTRY_REMOVED,
 	NON_ENTRY_DROPPED,
 	POST_ENTRY_DELETE,
+	FILTER_ENTRY_DROP,
 	LAST_SIGNAL
 };
 
@@ -395,6 +404,25 @@ rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 			      g_cclosure_marshal_VOID__BOXED,
 			      G_TYPE_NONE,
 			      1, RHYTHMDB_TYPE_ENTRY);
+	/**
+	 * RhythmDBQueryModel::filter-entry-drop:
+	 * @model: the #RhythmDBQueryModel
+	 * @entry: the #RhythmDBEntry being dropped
+	 *
+	 * Emitted when an entry is being added to a model by drag-and-drop.
+	 * This allows the owner of the model to filter out entries that should
+	 * not be added to the model (based on entry type, for instance).
+	 * If the signal handler returns %FALSE, the entry will not be added.
+	 */
+	rhythmdb_query_model_signals[FILTER_ENTRY_DROP] =
+		g_signal_new ("filter-entry-drop",
+			      RHYTHMDB_TYPE_QUERY_MODEL,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RhythmDBQueryModelClass, filter_entry_drop),
+			      NULL, NULL,
+			      rb_marshal_BOOLEAN__BOXED,
+			      G_TYPE_BOOLEAN,
+			      1, RHYTHMDB_TYPE_ENTRY);
 
 	g_type_class_add_private (klass, sizeof (RhythmDBQueryModelPrivate));
 }
@@ -1489,6 +1517,63 @@ rhythmdb_query_model_filter_out_entry (RhythmDBQueryModel *model,
 	}
 }
 
+void
+rhythmdb_query_model_shuffle_entries (RhythmDBQueryModel *model)
+{
+	RhythmDBEntry **entries;
+	int listsize;
+	int i;
+	int swapwith;
+	RhythmDBEntry *entry;
+	GSequenceIter *iter;
+	GtkTreePath *path;
+	GtkTreeIter tree_iter;
+	int *map_new_old;
+
+	/* Convert the entries list to an array, for fast lookups. */
+	listsize = g_sequence_get_length (model->priv->entries);
+	entries = (RhythmDBEntry **)g_malloc (sizeof(RhythmDBEntry *) * listsize);
+	map_new_old = (int *)g_malloc (sizeof(int) * listsize);
+
+	iter = g_sequence_get_begin_iter (model->priv->entries);
+	i = 0;
+	while (!g_sequence_iter_is_end (iter)) {
+		entries[i++] = g_sequence_get (iter);
+		iter = g_sequence_iter_next (iter);
+	}
+
+	/* Shuffle the array. */
+	for (i=0; i<listsize; i++) {
+		swapwith = g_random_int_range (i, listsize);
+		map_new_old[swapwith] = i;
+		entry = entries[swapwith];
+		entries[swapwith] = entries[i];
+		entries[i] = entry;
+	}
+
+	/* Convert the array back into a sequence, rebuilding the reverse map. */
+	iter = g_sequence_get_begin_iter (model->priv->entries);
+	i = 0;
+	while (!g_sequence_iter_is_end (iter)) {
+		g_sequence_set (iter, (gpointer)entries[i]);
+		rhythmdb_entry_ref (entries[i]);
+		g_hash_table_remove (model->priv->reverse_map, entries[i]);
+		g_hash_table_insert (model->priv->reverse_map, entries[i], iter);
+
+		iter = g_sequence_iter_next (iter);
+		i++;
+	}
+
+	/* Emit reorder signals. */
+	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &tree_iter);
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &tree_iter);
+	gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model), path, &tree_iter, map_new_old);
+	gtk_tree_path_free (path);
+
+	g_free (map_new_old);
+	g_free (entries);
+}
+
 void
 rhythmdb_query_model_move_entry (RhythmDBQueryModel *model,
 				 RhythmDBEntry *entry,
@@ -1824,8 +1909,20 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 
 				old_ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
 				/* trying to drag drop an entry on itself ! */
-				if (old_ptr == ptr)
+				if (old_ptr == ptr) {
 					continue;
+				} else if (old_ptr == NULL) {
+					gboolean allow;
+
+					g_signal_emit (G_OBJECT (model),
+						       rhythmdb_query_model_signals[FILTER_ENTRY_DROP],
+						       0, entry, &allow);
+					if (allow == FALSE) {
+						rb_debug ("dropping of entry %s disallowed by filter",
+							  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+						continue;
+					}
+				}
 
 				/* take temporary ref */
 				rhythmdb_entry_ref (entry);
@@ -1872,14 +1969,13 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 	return FALSE;
 }
 
+/* 
+ * determines whether reordering is possible by checking up
+ * the chain for a model with a sort function set.
+ */
 static gboolean
-rhythmdb_query_model_row_drop_possible (RbTreeDragDest *drag_dest,
-					GtkTreePath *dest,
-					GtkTreeViewDropPosition pos,
-					GtkSelectionData *selection_data)
+query_model_chain_can_reorder (RhythmDBQueryModel *model)
 {
-	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (drag_dest);
-
 	while (model) {
 		if (model->priv->sort_func != NULL)
 			return FALSE;
@@ -1889,13 +1985,24 @@ rhythmdb_query_model_row_drop_possible (RbTreeDragDest *drag_dest,
 	return TRUE;
 }
 
+static gboolean
+rhythmdb_query_model_row_drop_possible (RbTreeDragDest *drag_dest,
+					GtkTreePath *dest,
+					GtkTreeViewDropPosition pos,
+					GtkSelectionData *selection_data)
+{
+	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (drag_dest);
+	return query_model_chain_can_reorder (model);
+}
+
 static gboolean
 rhythmdb_query_model_row_drop_position (RbTreeDragDest *drag_dest,
 					GtkTreePath *dest_path,
 					GList *targets,
 					GtkTreeViewDropPosition *pos)
 {
-	return TRUE;
+	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (drag_dest);
+	return query_model_chain_can_reorder (model);
 }
 
 static void
