@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -107,7 +115,9 @@ static void rhythmdb_hash_tree_foreach (RhythmDB *adb,
 					RBTreePropertyItFunc genres_func,
 					gpointer data);
 
-#define RHYTHMDB_TREE_XML_VERSION "1.3"
+/* Update both of those! */
+#define RHYTHMDB_TREE_XML_VERSION "1.4"
+#define RHYTHMDB_TREE_XML_VERSION_INT 140
 
 static void destroy_tree_property (RhythmDBTreeProperty *prop);
 static RhythmDBTreeProperty *get_or_create_album (RhythmDBTree *db, RhythmDBTreeProperty *artist,
@@ -326,6 +336,20 @@ struct RhythmDBTreeLoadContext
 	gboolean reload_all_metadata;
 };
 
+/* Returns the version as an int, multiplied by 100,
+ * eg. "1.4" becomes 140 */
+static int
+version_to_int (const char *version)
+{
+	float ver;
+
+	if (sscanf (version, "%f", &ver) != 1) {
+		return (int) (1.0 * 100);
+	}
+
+	return ver * 100;
+}
+
 static void
 rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 				    const char *name,
@@ -351,22 +375,32 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 				if (!strcmp (*attrs, "version")) {
 					const char *version = *(attrs+1);
 
-					if (!strcmp (version, "1.0") || !strcmp (version, "1.1")) {
+					switch (version_to_int (version)) {
+					case 100:
+					case 110:
+						rb_debug ("old version of rhythmdb, performing URI canonicalisation for all entries (DB version 1.0 or 1.1)");
 						ctx->canonicalise_uris = TRUE;
-						rb_debug ("old version of rhythmdb, performing URI canonicalisation for all entries");
-					} else if (!strcmp (version, "1.2")) {
-						/* current version*/
-						rb_debug ("reloading all file metadata to get MusicBrainz tags");
+					case 120:
+						rb_debug ("reloading all file metadata to get MusicBrainz tags (DB version 1.2)");
 						ctx->reload_all_metadata = TRUE;
-					} else if (!strcmp (version, "1.3")) {
-						/* current version*/
-					} else {
-						g_set_error (ctx->error,
-							     RHYTHMDB_TREE_ERROR,
-							     RHYTHMDB_TREE_ERROR_DATABASE_TOO_NEW,
-							     _("The database was created by a later version of rhythmbox."
-							       "  This version of rhythmbox cannot read the database."));
-						xmlStopParser (ctx->xmlctx);
+					case 130:
+						/* Avoid being warned twice for very old DBs */
+						if (ctx->canonicalise_uris == FALSE) {
+							rb_debug ("old version of rhythmdb, performing URI canonicalisation for all entries (DB version 1.3)");
+							ctx->canonicalise_uris = TRUE;
+						}
+					case RHYTHMDB_TREE_XML_VERSION_INT:
+						/* current version */
+						break;
+					default:
+						if (version_to_int (version) > RHYTHMDB_TREE_XML_VERSION_INT) {
+							g_set_error (ctx->error,
+								     RHYTHMDB_TREE_ERROR,
+								     RHYTHMDB_TREE_ERROR_DATABASE_TOO_NEW,
+								     _("The database was created by a later version of rhythmbox."
+								       "  This version of rhythmbox cannot read the database."));
+							xmlStopParser (ctx->xmlctx);
+						}
 					}
 				} else {
 					g_assert_not_reached ();
@@ -899,8 +933,8 @@ save_entry (RhythmDBTree *db,
 		case RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID:
 			save_entry_string(ctx, elt_name, rb_refstring_get (entry->musicbrainz_albumartistid));
 			break;
-		case RHYTHMDB_PROP_MUSICBRAINZ_ARTISTSORTNAME:
-			save_entry_string(ctx, elt_name, rb_refstring_get (entry->musicbrainz_artistsortname));
+		case RHYTHMDB_PROP_ARTIST_SORTNAME:
+			save_entry_string(ctx, elt_name, rb_refstring_get (entry->artist_sortname));
 			break;
 		case RHYTHMDB_PROP_TRACK_NUMBER:
 			save_entry_ulong (ctx, elt_name, entry->tracknum, FALSE);
