@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -29,6 +37,18 @@
 
 #include "rhythmdb.h"
 
+/**
+ * SECTION:rb-history
+ * @short_description: sequence data structure useful for implementing play orders
+ *
+ * RBHistory is a GSequence that maintains a "current" pointer and can delete
+ * an arbitrary element in amortized O(log(N)) time. It can call a deletion
+ * callback when it removes one of its entries.
+ *
+ * All operations take amortized O(log(N)) (worst-case O(N)) time unless noted
+ * otherwise.
+ */
+
 struct RBHistoryPrivate
 {
 	GSequence *seq;
@@ -86,6 +106,11 @@ rb_history_class_init (RBHistoryClass *klass)
 	object_class->set_property = rb_history_set_property;
 	object_class->get_property = rb_history_get_property;
 
+	/**
+	 * RBHistory:truncate-on-play:
+	 *
+	 * If set, rb_history_set_playing() truncates the rest of the history
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_TRUNCATE_ON_PLAY,
 					 g_param_spec_boolean ("truncate-on-play",
@@ -94,6 +119,11 @@ rb_history_class_init (RBHistoryClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
+	/**
+	 * RBHistory:maximum-size:
+	 *
+	 * Maximum number of entries to store in the history.  If 0, no limit is applied.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_MAX_SIZE,
 					 g_param_spec_uint ("maximum-size",
@@ -106,6 +136,14 @@ rb_history_class_init (RBHistoryClass *klass)
 	g_type_class_add_private (klass, sizeof (RBHistoryPrivate));
 }
 
+/**
+ * rb_history_new:
+ * @truncate_on_play: Whether rb_history_set_playing() should truncate the history
+ * @destroyer: function to call when removing an entry from the history
+ * @destroy_userdata: data to pass to @destroyer
+ *
+ * Return value: a new #RBHistory
+ */
 RBHistory *
 rb_history_new (gboolean truncate_on_play, GFunc destroyer, gpointer destroy_userdata)
 {
@@ -198,6 +236,14 @@ rb_history_get_property (GObject *object,
 
 }
 
+/**
+ * rb_history_set_destroy_notify:
+ * @hist: a #RBHistory
+ * @destroyer: function to call when removing an entry from the history
+ * @destroy_userdata: data to pass to @destroyer
+ *
+ * Sets a new function to call when removing entries from the history.
+ */
 void
 rb_history_set_destroy_notify (RBHistory *hist, GFunc destroyer, gpointer destroy_userdata)
 {
@@ -207,6 +253,13 @@ rb_history_set_destroy_notify (RBHistory *hist, GFunc destroyer, gpointer destro
 	hist->priv->destroy_userdata = destroy_userdata;
 }
 
+/**
+ * rb_history_set_truncate_on_play:
+ * @hist: a #RBHistory
+ * @truncate_on_play: Whether rb_history_set_playing() should truncate the history
+ *
+ * Sets the 'truncate-on-play' property.
+ */
 void
 rb_history_set_truncate_on_play (RBHistory *hist, gboolean truncate_on_play)
 {
@@ -216,6 +269,13 @@ rb_history_set_truncate_on_play (RBHistory *hist, gboolean truncate_on_play)
 	g_object_notify (G_OBJECT (hist), "truncate-on-play");
 }
 
+/**
+ * rb_history_set_maximum_size:
+ * @hist: a #RBHistory
+ * @maximum_size: new maximum size of the history (or 0 for no limit)
+ *
+ * Sets the maximum-size property
+ */
 void
 rb_history_set_maximum_size (RBHistory *hist, guint maximum_size)
 {
@@ -225,6 +285,12 @@ rb_history_set_maximum_size (RBHistory *hist, guint maximum_size)
 	g_object_notify (G_OBJECT (hist), "maximum-size");
 }
 
+/**
+ * rb_history_length:
+ * @hist: a #RBHistory
+ *
+ * Return value: the number of entries in the history
+ */
 guint
 rb_history_length (RBHistory *hist)
 {
@@ -233,6 +299,12 @@ rb_history_length (RBHistory *hist)
 	return g_sequence_get_length (hist->priv->seq);
 }
 
+/**
+ * rb_history_first:
+ * @hist: a #RBHistory
+ *
+ * Return value: the first entry in the history
+ */
 RhythmDBEntry *
 rb_history_first (RBHistory *hist)
 {
@@ -243,6 +315,12 @@ rb_history_first (RBHistory *hist)
 	return g_sequence_iter_is_end (begin) ? NULL : g_sequence_get (begin);
 }
 
+/**
+ * rb_history_previous:
+ * @hist: a #RBHistory
+ *
+ * Return value: the #RhythmDBEntry before the current position
+ */
 RhythmDBEntry *
 rb_history_previous (RBHistory *hist)
 {
@@ -254,6 +332,12 @@ rb_history_previous (RBHistory *hist)
 	return prev == hist->priv->current ? NULL : g_sequence_get (prev);
 }
 
+/**
+ * rb_history_current:
+ * @hist: a #RBHistory
+ *
+ * Return value: the current #RhythmDBEntry, or NULL if there is no current position
+ */
 RhythmDBEntry *
 rb_history_current (RBHistory *hist)
 {
@@ -262,6 +346,12 @@ rb_history_current (RBHistory *hist)
 	return g_sequence_iter_is_end (hist->priv->current) ? NULL : g_sequence_get (hist->priv->current);
 }
 
+/**
+ * rb_history_next:
+ * @hist: a #RBHistory
+ *
+ * Return value: the #RhythmDBEntry after the current position
+ */
 RhythmDBEntry *
 rb_history_next (RBHistory *hist)
 {
@@ -272,6 +362,12 @@ rb_history_next (RBHistory *hist)
 	return g_sequence_iter_is_end (next) ? NULL : g_sequence_get (next);
 }
 
+/**
+ * rb_history_last:
+ * @hist: a #RBHistory
+ *
+ * Return value: the last #RhythmDBEntry in the history
+ */
 RhythmDBEntry *
 rb_history_last (RBHistory *hist)
 {
@@ -283,6 +379,12 @@ rb_history_last (RBHistory *hist)
 	return g_sequence_iter_is_end (last) ? NULL : g_sequence_get (last);
 }
 
+/**
+ * rb_history_go_first:
+ * @hist: a #RBHistory
+ *
+ * Moves the current position to the first entry in the history
+ */
 void
 rb_history_go_first (RBHistory *hist)
 {
@@ -291,6 +393,13 @@ rb_history_go_first (RBHistory *hist)
 	hist->priv->current = g_sequence_get_begin_iter (hist->priv->seq);
 }
 
+/**
+ * rb_history_go_previous:
+ * @hist: a #RBHistory
+ *
+ * Moves the current position to the previous entry.  If the current position is
+ * already at the start of the history, nothing happens.
+ */
 void
 rb_history_go_previous (RBHistory *hist)
 {
@@ -302,6 +411,13 @@ rb_history_go_previous (RBHistory *hist)
 		hist->priv->current = prev;
 }
 
+/**
+ * rb_history_go_next:
+ * @hist: a #RBHistory
+ *
+ * Moves the current position to the next entry.  If the current position is
+ * already at the end of the history, nothing happens.
+ */
 void
 rb_history_go_next (RBHistory *hist)
 {
@@ -310,6 +426,12 @@ rb_history_go_next (RBHistory *hist)
 	hist->priv->current = g_sequence_iter_next (hist->priv->current);
 }
 
+/**
+ * rb_history_go_last:
+ * @hist: a #RBHistory
+ *
+ * Moves the current position to the last entry in the history
+ */
 void
 rb_history_go_last (RBHistory *hist)
 {
@@ -326,6 +448,15 @@ _history_remove_swapped (RhythmDBEntry *entry, RBHistory *hist)
 	rb_history_remove_entry_internal (hist, entry, FALSE);
 }
 
+/**
+ * rb_history_set_playing:
+ * @hist: a #RBHistory
+ * @entry: the new playing #RhythmDBEntry
+ *
+ * Updates the current position to point to the specified entry.
+ * If the truncate-on-play property is set, this will remove all entries
+ * after that.
+ */
 void
 rb_history_set_playing (RBHistory *hist, RhythmDBEntry *entry)
 {
@@ -357,6 +488,15 @@ rb_history_set_playing (RBHistory *hist, RhythmDBEntry *entry)
 	rb_history_limit_size (hist, TRUE);
 }
 
+/**
+ * rb_history_append:
+ * @hist: a #RBHistory
+ * @entry: a #RhythmDBEntry to append
+ *
+ * Adds a new entry to the end of the history list.
+ * If a size limit is set, an entry may be removed from the start to
+ * keep the history list within the limit.
+ */
 void
 rb_history_append (RBHistory *hist, RhythmDBEntry *entry)
 {
@@ -382,6 +522,15 @@ rb_history_append (RBHistory *hist, RhythmDBEntry *entry)
 	rb_history_limit_size (hist, TRUE);
 }
 
+/**
+ * rb_history_get_current_index:
+ * @hist: a #RBHistory
+ *
+ * Gets the index of the current entry. This is guaranteed to be < the
+ * history's size, so if the history is empty, it returns -1.
+ *
+ * Return value: index of the current entry
+ */
 gint
 rb_history_get_current_index (RBHistory *hist)
 {
@@ -390,6 +539,14 @@ rb_history_get_current_index (RBHistory *hist)
 	return g_sequence_iter_get_position (hist->priv->current);
 }
 
+/**
+ * rb_history_insert_at_index:
+ * @hist: a #RBHistory
+ * @entry: a #RhythmDBEntry to insert
+ * @index: position at which to insert @entry
+ *
+ * Inserts @entry at @index within the history list. 0<=@index<=size
+ */
 void
 rb_history_insert_at_index (RBHistory *hist, RhythmDBEntry *entry, guint index)
 {
@@ -418,14 +575,14 @@ rb_history_insert_at_index (RBHistory *hist, RhythmDBEntry *entry, guint index)
 	rb_history_limit_size (hist, TRUE);
 }
 
-/**
+/*
  * Cuts nodes off of the history from the desired end until it is smaller than max_size.
  * Never cuts off the current node.
  */
 static void
 rb_history_limit_size (RBHistory *hist, gboolean cut_from_beginning)
 {
-	if (hist->priv->maximum_size != 0)
+	if (hist->priv->maximum_size != 0) {
 		while (g_sequence_get_length (hist->priv->seq) > hist->priv->maximum_size) {
 			if (cut_from_beginning
 					|| hist->priv->current == g_sequence_iter_prev (g_sequence_get_end_iter (hist->priv->seq))) {
@@ -434,8 +591,16 @@ rb_history_limit_size (RBHistory *hist, gboolean cut_from_beginning)
 				rb_history_remove_entry (hist, rb_history_last (hist));
 			}
 		}
+	}
 }
 
+/**
+ * rb_history_remove_entry:
+ * @hist: a #RBHistory
+ * @entry: the #RhythmDBEntry to remove
+ *
+ * Removes the specified entry from the history list.
+ */
 void
 rb_history_remove_entry (RBHistory *hist, RhythmDBEntry *entry)
 {
@@ -464,6 +629,12 @@ rb_history_remove_entry_internal (RBHistory *hist, RhythmDBEntry *entry, gboolea
 	}
 }
 
+/**
+ * rb_history_clear:
+ * @hist: a #RBHistory
+ *
+ * Empties the history list.
+ */
 void
 rb_history_clear (RBHistory *hist)
 {
@@ -477,6 +648,16 @@ rb_history_clear (RBHistory *hist)
 	g_assert (g_hash_table_size (hist->priv->entry_to_seqptr) == 0);
 }
 
+/**
+ * rb_history_dump:
+ * @hist: a #RBHistory
+ *
+ * Constructs a copy of the whole history in order. Caller must free the result.
+ * The caller does not own any references on the entries in the returned array.
+ * Takes O(Nlog(N)) time.
+ *
+ * Return value: a copy of the history list
+ */
 GPtrArray *
 rb_history_dump (RBHistory *hist)
 {
@@ -494,6 +675,13 @@ rb_history_dump (RBHistory *hist)
 	return result;
 }
 
+/**
+ * rb_history_contains_entry:
+ * @hist: a #RBHistory
+ * @entry: a #RhythmDBEntry to check for
+ *
+ * Return value: TRUE if the entry is present in the history list.
+ */
 gboolean
 rb_history_contains_entry (RBHistory *hist, RhythmDBEntry *entry)
 {
