@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -33,15 +41,25 @@
 #include "rb-preferences.h"
 #include "rb-marshal.h"
 
-/* Play Orders */
-#include "rb-play-order-linear.h"
-#include "rb-play-order-linear-loop.h"
-#include "rb-play-order-shuffle.h"
-#include "rb-play-order-random-equal-weights.h"
-#include "rb-play-order-random-by-age.h"
-#include "rb-play-order-random-by-rating.h"
-#include "rb-play-order-random-by-age-and-rating.h"
-#include "rb-play-order-queue.h"
+/**
+ * SECTION:rb-play-order
+ * @short_description: base class for play order implementations
+ *
+ * A play order defines an ordering of the entries from a #RhythmDBQueryModel that
+ * the #RBShellPlayer uses to get the next or previous entry to play.
+ *
+ * Play order methods are invoked when changes are made to the query model, when
+ * the query model is replaced, the playing source is changed, or a new playing
+ * entry is selected.
+ *
+ * The play order must implement methods to check for, retrieve, and move to the
+ * next and previous entries in the play order.  Only the go_next and go_previous
+ * methods actually change anything.
+ *
+ * The play order should also emit the have-next-previous-changed signal to hint that
+ * the availability of either a next or previous entry in the order may have changed.
+ * This information is used to update the sensitivity of the next and previous buttons.
+ */
 
 static void rb_play_order_class_init (RBPlayOrderClass *klass);
 static void rb_play_order_init (RBPlayOrder *porder);
@@ -121,6 +139,11 @@ rb_play_order_class_init (RBPlayOrderClass *klass)
 	klass->has_previous = default_has_previous;
 	klass->playing_entry_removed = default_playing_entry_removed;
 
+	/**
+	 * RBPlayOrder:player:
+	 *
+	 * The #RBShellPlayer instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_PLAYER,
 					 g_param_spec_object ("player",
@@ -129,6 +152,11 @@ rb_play_order_class_init (RBPlayOrderClass *klass)
 						 	      RB_TYPE_SHELL_PLAYER,
 						 	      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	 * RBPlayOrder:playing-entry:
+	 *
+	 * The current playing #RhythmDBEntry
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_PLAYING_ENTRY,
 					 g_param_spec_boxed ("playing-entry",
@@ -137,6 +165,16 @@ rb_play_order_class_init (RBPlayOrderClass *klass)
 							     RHYTHMDB_TYPE_ENTRY,
 						 	     G_PARAM_READWRITE));
 
+	/**
+	 * RBPlayOrder::have-next-previous-changed:
+	 * @porder: the #RBPlayOrder
+	 * @have_next: if %TRUE, the play order has at least one more entry
+	 * @have_previous: if %TRUE, the play order has at least one entry
+	 *    before the current entry
+	 *
+	 * Emitted as a hint to suggest that the sensitivity of next/previous
+	 * buttons may need to be updated.
+	 */
 	rb_play_order_signals[HAVE_NEXT_PREVIOUS_CHANGED] =
 		g_signal_new ("have_next_previous_changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -289,72 +327,6 @@ rb_play_order_get_property (GObject *object,
 	}
 }
 
-/**
- * rb_play_order_get_orders:
- *
- * Defines the set of available play orders, their translatable descriptions, their
- * constructor functions, whether they should appear in a drop-down list of
- * play orders, and which one is the default.
- *
- * This should be the only function with full knowledge of what play orders are
- * available.
- */
-const RBPlayOrderDescription *
-rb_play_order_get_orders (void)
-{
-	/* Exactly one entry must have is_default==TRUE. Otherwise you will
-	 * cause a g_assert(). */
-	static const RBPlayOrderDescription orders[] = {
-		{ "linear", N_("Linear"), rb_linear_play_order_new, TRUE, TRUE },
-		{ "linear-loop", N_("Linear looping"), rb_linear_play_order_loop_new, TRUE, FALSE },
-		{ "shuffle", N_("Shuffle"), rb_shuffle_play_order_new, TRUE, FALSE },
-		{ "random-equal-weights", N_("Random with equal weights"), rb_random_play_order_equal_weights_new, TRUE, FALSE },
-		{ "random-by-age", N_("Random by time since last play"), rb_random_play_order_by_age_new, TRUE, FALSE },
-		{ "random-by-rating", N_("Random by rating"), rb_random_play_order_by_rating_new, TRUE, FALSE },
-		{ "random-by-age-and-rating", N_("Random by time since last play and rating"), rb_random_play_order_by_age_and_rating_new, TRUE, FALSE },
-		{ "queue", N_("Linear, removing entries once played"), rb_queue_play_order_new, FALSE, FALSE },
-		{ NULL, NULL, NULL },
-	};
-	return orders;
-}
-
-/**
- * rb_play_order_new:
- * @porder_name: Play order type name
- * @player: #RBShellPlayer instance to attach to
- *
- * Creates a new #RBPlayOrder of the specified type.
- *
- * Returns: #RBPlayOrder instance
- **/
-RBPlayOrder *
-rb_play_order_new (const char* porder_name,
-		   RBShellPlayer *player)
-{
-	int default_index = -1;
-	const RBPlayOrderDescription *orders = rb_play_order_get_orders ();
-	int i;
-
-	g_return_val_if_fail (porder_name != NULL, NULL);
-	g_return_val_if_fail (player != NULL, NULL);
-
-	for (i=0; orders[i].name != NULL; ++i) {
-		if (strcmp (orders[i].name, porder_name) == 0)
-			return orders[i].constructor (player);
-		if (orders[i].is_default) {
-			/* There must not be two default play orders */
-			g_assert (default_index == -1);
-			default_index = i;
-		}
-	}
-	/* There must be a default play order */
-	g_assert (default_index != -1);
-
-	g_warning ("Unknown value \"%s\" in GConf key \"" CONF_STATE_PLAY_ORDER
-			"\". Using %s play order.", porder_name, orders[default_index].name);
-	return orders[default_index].constructor (player);
-}
-
 /**
  * rb_play_order_get_player:
  * @porder: #RBPlayOrder instance
@@ -735,7 +707,7 @@ default_playing_entry_removed (RBPlayOrder *porder,
 		case RB_SOURCE_EOF_STOP:
 		case RB_SOURCE_EOF_RETRY:
 			/* stop playing */
-			rb_shell_player_set_playing_source (player, NULL);
+			rb_shell_player_stop (player);
 			break;
 		case RB_SOURCE_EOF_NEXT:
 			{
