@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,6 +28,15 @@
  *
  */
 
+/**
+ * SECTION:rb-playlist-manager
+ * @short_description: Playlist management object
+ *
+ * The playlist manager loads and saves the on-disk playlist file, provides
+ * UI actions and a DBus interface for dealing with playlists, and internal
+ * interfaces for creating playlists.
+ */
+
 #include "config.h"
 
 #include <string.h>
@@ -74,6 +91,8 @@ static void rb_playlist_manager_cmd_new_playlist (GtkAction *action,
 						  RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
 							    RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_shuffle_playlist (GtkAction *action,
+						      RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_rename_playlist (GtkAction *action,
 						     RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_delete_playlist (GtkAction *action,
@@ -130,10 +149,10 @@ static GtkActionEntry rb_playlist_manager_actions [] =
 	/* Submenu of Music */
 	{ "Playlist", NULL, N_("_Playlist") },
 
-	{ "MusicPlaylistNewPlaylist", RB_STOCK_PLAYLIST, N_("_New Playlist"), "<control>N",
+	{ "MusicPlaylistNewPlaylist", RB_STOCK_PLAYLIST_NEW, N_("_New Playlist..."), "<control>N",
 	  N_("Create a new playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_new_playlist) },
-	{ "MusicPlaylistNewAutomaticPlaylist", RB_STOCK_AUTO_PLAYLIST, N_("New _Automatic Playlist..."), NULL,
+	{ "MusicPlaylistNewAutomaticPlaylist", RB_STOCK_AUTO_PLAYLIST_NEW, N_("New _Automatic Playlist..."), NULL,
 	  N_("Create a new automatically updating playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_new_automatic_playlist) },
 	{ "MusicPlaylistLoadPlaylist", NULL, N_("_Load from File..."), NULL,
@@ -154,6 +173,9 @@ static GtkActionEntry rb_playlist_manager_actions [] =
 	{ "QueuePlaylist", NULL, N_("_Queue All Tracks"), NULL,
 	  N_("Add all tracks in this playlist to the queue"),
 	  G_CALLBACK (rb_playlist_manager_cmd_queue_playlist) },
+	{ "ShufflePlaylist", NULL, N_("_Shuffle Playlist"), NULL,
+	  N_("Shuffle the tracks in this playlist"),
+	  G_CALLBACK (rb_playlist_manager_cmd_shuffle_playlist) },
 };
 static guint rb_playlist_manager_n_actions = G_N_ELEMENTS (rb_playlist_manager_actions);
 
@@ -201,6 +223,14 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 							      "RBSourceList",
 							      RB_TYPE_SOURCELIST,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBPlaylistManager::playlist-added:
+	 * @manager: the #RBPlaylistManager
+	 * @source: the new #RBSource
+	 *
+	 * Emitted when a playlist is added, including when being loaded
+	 * from the user's playlist file.
+	 */
 	rb_playlist_manager_signals[PLAYLIST_ADDED] =
 		g_signal_new ("playlist_added",
 			      RB_TYPE_PLAYLIST_MANAGER,
@@ -211,6 +241,13 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      G_TYPE_NONE,
 			      1, G_TYPE_OBJECT);
 
+	/**
+	 * RBPlaylistManager::playlist-created:
+	 * @manager: the #RBPlaylistManager
+	 * @source: the newly created playlist #RBSource
+	 *
+	 * Emitted when a new playlist is created.
+	 */
 	rb_playlist_manager_signals[PLAYLIST_CREATED] =
 		g_signal_new ("playlist_created",
 			      RB_TYPE_PLAYLIST_MANAGER,
@@ -221,6 +258,13 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      G_TYPE_NONE,
 			      1, G_TYPE_OBJECT);
 
+	/**
+	 * RBPlaylistManager::load-start:
+	 * @manager: the #RBPlaylistManager
+	 *
+	 * Emitted when the playlist manager starts loading the user's
+	 * playlist file.
+	 */
 	rb_playlist_manager_signals[PLAYLIST_LOAD_START] =
 		g_signal_new ("load_start",
 			      RB_TYPE_PLAYLIST_MANAGER,
@@ -230,6 +274,13 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0, G_TYPE_NONE);
+	/**
+	 * RBPlaylistManager::load-finish
+	 * @manager: the #RBPlaylistManager
+	 *
+	 * Emitted when the playlist manager finishes loading the user's
+	 * playlist file.
+	 */
 	rb_playlist_manager_signals[PLAYLIST_LOAD_FINISH] =
 		g_signal_new ("load_finish",
 			      RB_TYPE_PLAYLIST_MANAGER,
@@ -255,6 +306,13 @@ rb_playlist_manager_init (RBPlaylistManager *mgr)
 	mgr->priv->saving = 0;
 }
 
+/**
+ * rb_playlist_manager_shutdown:
+ * @mgr: the #RBPlaylistManager
+ *
+ * Shuts down the playlist manager, making sure any outstanding playlist save
+ * operation finishes.
+ */
 void
 rb_playlist_manager_shutdown (RBPlaylistManager *mgr)
 {
@@ -362,6 +420,7 @@ rb_playlist_manager_set_source (RBPlaylistManager *mgr,
 	gboolean can_delete;
 	gboolean can_edit;
 	gboolean can_rename;
+	gboolean can_shuffle;
 	GtkAction *action;
 
 	party_mode = rb_shell_get_party_mode (mgr->priv->shell);
@@ -397,6 +456,11 @@ rb_playlist_manager_set_source (RBPlaylistManager *mgr,
 	action = gtk_action_group_get_action (mgr->priv->actiongroup,
 					      "MusicPlaylistRenamePlaylist");
 	gtk_action_set_visible (action, can_rename);
+
+	can_shuffle = RB_IS_STATIC_PLAYLIST_SOURCE (mgr->priv->selected_source);
+	action = gtk_action_group_get_action (mgr->priv->actiongroup,
+					      "ShufflePlaylist");
+	gtk_action_set_sensitive (action, can_shuffle);
 }
 
 static void
@@ -480,6 +544,16 @@ rb_playlist_manager_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_playlist_manager_new:
+ * @shell: the #RBShell
+ * @sourcelist: the #RBSourceList
+ * @playlists_file: the full path to the playlist file to load
+ *
+ * Creates the #RBPlaylistManager instance
+ *
+ * Return value: the #RBPlaylistManager
+ */
 RBPlaylistManager *
 rb_playlist_manager_new (RBShell *shell,
 			 RBSourceList *sourcelist,
@@ -502,28 +576,17 @@ rb_playlist_manager_error_quark (void)
 	return quark;
 }
 
-#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 static void
 handle_playlist_entry_cb (TotemPlParser *playlist,
 			  const char *uri_maybe,
 			  GHashTable *metadata,
 			  RBPlaylistManager *mgr)
-#else
-static void
-handle_playlist_entry_cb (TotemPlParser *playlist,
-			  const char *uri_maybe,
-			  const char *title,
-			  const char *genre,
-			  RBPlaylistManager *mgr)
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 {
 	char *uri;
-#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 	const char *title, *genre;
 
 	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
 	genre = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_GENRE);
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
 	uri = rb_canonicalise_uri (uri_maybe);
 	g_return_if_fail (uri != NULL);
@@ -550,25 +613,20 @@ handle_playlist_entry_cb (TotemPlParser *playlist,
 }
 
 static void
-playlist_load_start_cb (TotemPlParser *parser, const char *title, RBPlaylistManager *mgr)
+playlist_load_started_cb (TotemPlParser *parser, const char *uri, GHashTable *metadata, RBPlaylistManager *mgr)
 {
-	rb_debug ("loading new playlist %s", title);
+	const char *title;
+
+	rb_debug ("loading new playlist %s", uri);
+
+	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
+	if (title == NULL)
+		title = _("Unnamed playlist");
 
 	mgr->priv->loading_playlist =
 			RB_STATIC_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, title, FALSE));
 }
 
-static void
-playlist_load_end_cb (TotemPlParser *parser, const char *title, RBPlaylistManager *mgr)
-{
-	rb_debug ("finished loading playlist %s", title);
-
-	if (title) {
-		g_object_set (mgr->priv->loading_playlist, "name", title, NULL);
-		mgr->priv->loading_playlist = NULL;
-	}
-}
-
 /**
  * rb_playlist_manager_parse_file:
  * @mgr: the #RBPlaylistManager
@@ -579,7 +637,7 @@ playlist_load_end_cb (TotemPlParser *parser, const char *title, RBPlaylistManage
  * static playlist.  If the playlist file includes a title, the static
  * playlist created will have the same title.
  *
- * Returns TRUE on success
+ * Return value: TRUE on success
  **/
 gboolean
 rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError **error)
@@ -591,26 +649,15 @@ rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError
 	{
 		TotemPlParser *parser = totem_pl_parser_new ();
 
-#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 		g_signal_connect_object (parser, "entry-parsed",
 					 G_CALLBACK (handle_playlist_entry_cb),
 					 mgr, 0);
-#else
-		g_signal_connect_object (parser, "entry",
-					 G_CALLBACK (handle_playlist_entry_cb),
-					 mgr, 0);
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
-		g_signal_connect_object (parser, "playlist-start",
-					 G_CALLBACK (playlist_load_start_cb),
+		g_signal_connect_object (parser, "playlist-started",
+					 G_CALLBACK (playlist_load_started_cb),
 					 mgr, 0);
 
-		g_signal_connect_object (parser, "playlist-end",
-					 G_CALLBACK (playlist_load_end_cb),
-					 mgr, 0);
-
-		if (g_object_class_find_property (G_OBJECT_GET_CLASS (parser), "recurse"))
-			g_object_set (parser, "recurse", FALSE, NULL);
+		g_object_set (parser, "recurse", FALSE, NULL);
 
 		if (totem_pl_parser_parse (parser, uri, TRUE) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 			g_set_error (error,
@@ -620,6 +667,7 @@ rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError
 				     _("The playlist file may be in an unknown format or corrupted."));
 			return FALSE;
 		}
+
 		if (mgr->priv->loading_playlist != NULL) {
 			char *name = NULL;
 
@@ -722,6 +770,47 @@ rb_playlist_manager_set_dirty (RBPlaylistManager *mgr, gboolean dirty)
 	g_atomic_int_compare_and_exchange (&mgr->priv->dirty, dirty == FALSE, dirty == TRUE);
 }
 
+static gboolean
+_is_dirty_playlist (GtkTreeModel *model,
+		    GtkTreePath *path,
+		    GtkTreeIter *iter,
+		    gboolean *dirty)
+{
+	RBSource *source;
+	gboolean local;
+	gboolean ret;
+
+	gtk_tree_model_get (model,
+			    iter,
+			    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+			    &source,
+			    -1);
+	if (source == NULL) {
+		return FALSE;
+	}
+	if (RB_IS_PLAYLIST_SOURCE (source) == FALSE) {
+		g_object_unref (source);
+		return FALSE;
+	}
+
+	g_object_get (source,
+		      "is-local", &local,
+		      NULL);
+	ret = FALSE;
+	if (local) {
+		gboolean pdirty;
+
+		g_object_get (source, "dirty", &pdirty, NULL);
+		if (pdirty) {
+			*dirty = TRUE;
+			ret = TRUE;
+		}
+	}
+	g_object_unref (source);
+
+	return ret;
+}
+
 /* returns TRUE if a playlist has been created, modified, or deleted since last save */
 static gboolean
 rb_playlist_manager_is_dirty (RBPlaylistManager *mgr)
@@ -729,41 +818,12 @@ rb_playlist_manager_is_dirty (RBPlaylistManager *mgr)
 	gboolean dirty = FALSE;
 	GtkTreeModel *fmodel;
 	GtkTreeModel *model;
-	GtkTreeIter iter;
 
 	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
-	if (gtk_tree_model_get_iter_first (model, &iter)) {
-		do {
-			RBSource *source;
-			gboolean local;
-
-			gtk_tree_model_get (model,
-					    &iter,
-					    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-					    &source,
-					    -1);
-			if (source == NULL) {
-				continue;
-			}
-			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE) {
-				g_object_unref (source);
-				continue;
-			}
-
-			g_object_get (source,
-				      "is-local", &local,
-				      NULL);
-			if (local) {
-				g_object_get (source, "dirty", &dirty, NULL);
-				if (dirty)
-					break;
-			}
-			g_object_unref (source);
-		} while (gtk_tree_model_iter_next (model, &iter));
-	}
+	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc) _is_dirty_playlist, &dirty);
 
 	if (!dirty)
 		dirty = g_atomic_int_get (&mgr->priv->dirty);
@@ -809,10 +869,10 @@ rb_playlist_manager_save_data (struct RBPlaylistManagerSaveData *data)
 }
 
 static gboolean
-model_foreach_func (GtkTreeModel *model,
-		    GtkTreePath  *path,
-		    GtkTreeIter  *iter,
-		    xmlNodePtr    root)
+save_playlist_cb (GtkTreeModel *model,
+		  GtkTreePath  *path,
+		  GtkTreeIter  *iter,
+		  xmlNodePtr    root)
 {
 	RBSource *source;
 	gboolean  local;
@@ -851,7 +911,7 @@ model_foreach_func (GtkTreeModel *model,
  * since the last time the playlists were saved, and no save operation is
  * currently taking place.
  *
- * Returns TRUE if a playlist save operation has been started
+ * Return value: TRUE if a playlist save operation has been started
  **/
 gboolean
 rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
@@ -883,7 +943,7 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
-	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)model_foreach_func, root);
+	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)save_playlist_cb, root);
 
 	/* mark clean here.  if the save fails, we'll mark it dirty again */
 	rb_playlist_manager_set_dirty (data->mgr, FALSE);
@@ -904,7 +964,7 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
  *
  * Creates a new playlist and adds it to the source list.
  *
- * Returns the new playlist object.
+ * Return value: the new playlist object.
  **/
 RBSource *
 rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
@@ -919,6 +979,7 @@ rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
 	else
 		playlist = rb_static_playlist_source_new (mgr->priv->shell,
 							  suggested_name,
+							  NULL,
 							  TRUE,
 							  RHYTHMDB_ENTRY_TYPE_SONG);
 
@@ -1036,7 +1097,7 @@ create_name_from_selection_data (RBPlaylistManager *mgr,
  * Used to implement playlist creation through drag and drop
  * to the source list.
  *
- * Returns the new playlist.
+ * Return value: the new playlist.
  **/
 RBSource *
 rb_playlist_manager_new_playlist_from_selection_data (RBPlaylistManager *mgr,
@@ -1241,6 +1302,17 @@ rb_playlist_manager_cmd_queue_playlist (GtkAction *action,
 	g_object_unref (model);
 }
 
+static void
+rb_playlist_manager_cmd_shuffle_playlist (GtkAction *action,
+					RBPlaylistManager *mgr)
+{
+	RhythmDBQueryModel *base_model;
+
+	g_object_get (mgr->priv->selected_source, "base-query-model", &base_model, NULL);
+	rhythmdb_query_model_shuffle_entries (base_model);
+	g_object_unref (base_model);
+}
+
 static void
 rb_playlist_manager_cmd_rename_playlist (GtkAction *action,
 					 RBPlaylistManager *mgr)
@@ -1490,17 +1562,45 @@ rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 	g_object_unref (xml);
 }
 
+static gboolean
+list_playlists_cb (GtkTreeModel *model,
+		   GtkTreePath  *path,
+		   GtkTreeIter  *iter,
+		   GList **playlists)
+{
+	RBSource *source;
+	gboolean  local;
+
+	gtk_tree_model_get (model,
+			    iter,
+			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source,
+			    -1);
+	if (source != NULL) {
+		if (RB_IS_PLAYLIST_SOURCE (source) && !RB_IS_PLAY_QUEUE_SOURCE (source)) {
+			
+			g_object_get (source, "is-local", &local, NULL);
+			if (local) {
+				*playlists = g_list_prepend (*playlists, source);
+			}
+		}
+		
+		g_object_unref (source);
+	}
+
+	return FALSE;
+}
+
+
 /**
  * rb_playlist_manager_get_playlists
  * @mgr: the #RBPlaylistManager
  *
- * Returns a #GList containing all local playlist source objects.
+ * Return value: a #GList containing all local playlist source objects.
  **/
 GList *
 rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
 {
 	GList *playlists = NULL;
-	GtkTreeIter iter;
 	GtkTreeModel *fmodel;
 	GtkTreeModel *model;
 
@@ -1508,36 +1608,8 @@ rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
-	if (gtk_tree_model_get_iter_first (model, &iter)) {
-		do {
-			RBSource *source;
-			gboolean local;
-
-			gtk_tree_model_get (model,
-					    &iter,
-					    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-					    &source,
-					    -1);
-			if (source == NULL) {
-				continue;
-			}
-			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE
-			    || RB_IS_PLAY_QUEUE_SOURCE (source) == TRUE) {
-				g_object_unref (source);
-				continue;
-			}
-
-			g_object_get (source, "is-local", &local,
-				      NULL);
-			if (local) {
-				playlists = g_list_prepend (playlists, source);
-			}
-
-			g_object_unref (source);
-		} while (gtk_tree_model_iter_next (model, &iter));
-	}
-
-	return playlists;
+	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)list_playlists_cb, &playlists);
+	return g_list_reverse (playlists);
 }
 
 /**
@@ -1549,99 +1621,85 @@ rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
  * Allocates and returns an array containing the names of all local
  * playlists.  This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_get_playlist_names (RBPlaylistManager *mgr,
 					gchar ***playlists,
 					GError **error)
 {
-	GtkTreeIter iter;
-	GtkTreeModel *fmodel;
-	GtkTreeModel *model;
+	GList *pl;
+	GList *t;
 	int i;
-
-	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
-	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
-	g_object_unref (fmodel);
-
-	if (!gtk_tree_model_get_iter_first (model, &iter)) {
-		*playlists = NULL;
-		return TRUE;
-	}
-
-	*playlists = g_new0 (char *, gtk_tree_model_iter_n_children (model, NULL) + 1);
+	
+	pl = rb_playlist_manager_get_playlists (mgr);
+	*playlists = g_new0 (char *, g_list_length (pl) + 1);
 	if (!*playlists)
 		return FALSE;
 
 	i = 0;
-	do {
-		RBSource *source;
-		char *source_name;
+	for (t = pl; t != NULL; t = t->next, i++) {
+		const char *name;
+		RBSource *source = (RBSource *)t->data;
 
-		gtk_tree_model_get (model, &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-				    &source,
-				    -1);
-		if (source == NULL) {
-			continue;
-		}
-		if (RB_IS_PLAYLIST_SOURCE (source) == FALSE
-		    || RB_IS_PLAY_QUEUE_SOURCE (source) == TRUE) {
-			g_object_unref (source);
-			continue;
-		}
-
-		g_object_get (source, "name", &source_name, NULL);
-		(*playlists)[i++] = source_name;
-
-		g_object_unref (source);
-
-	} while (gtk_tree_model_iter_next (model, &iter));
+		g_object_get (source, "name", &name, NULL);
+		(*playlists)[i] = g_strdup (name);
+	}
 
 	return TRUE;
 }
 
+typedef struct {
+	const char *name;
+	RBSource *source;
+} FindPlaylistData;
+
+static gboolean
+find_playlist_by_name_cb (GtkTreeModel *model,
+			  GtkTreePath  *path,
+			  GtkTreeIter  *iter,
+			  FindPlaylistData *data)
+{
+	RBSource *source;
+
+	gtk_tree_model_get (model,
+			    iter,
+			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source,
+			    -1);
+	if (source != NULL) {
+		if (RB_IS_PLAYLIST_SOURCE (source) && !RB_IS_PLAY_QUEUE_SOURCE (source)) {
+			char *name;
+			
+			g_object_get (source, "name", &name, NULL);
+			if (strcmp (name, data->name) == 0) {
+				data->source = source;
+			}
+			g_free (name);
+		}
+		
+		g_object_unref (source);
+	}
+
+	return (data->source != NULL);
+}
+
 static RBSource *
 _get_playlist_by_name (RBPlaylistManager *mgr,
 		       const char *name)
 {
-	GtkTreeIter iter;
 	GtkTreeModel *fmodel;
 	GtkTreeModel *model;
-	RBSource *playlist = NULL;
+	FindPlaylistData d;
 
 	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
-	if (!gtk_tree_model_get_iter_first (model, &iter))
-		return NULL;
+	d.name = name;
+	d.source = NULL;
 
-	do {
-		RBSource *source;
-		char *source_name;
-
-		gtk_tree_model_get (model, &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-				    &source,
-				    -1);
-		if (source == NULL) {
-			continue;
-		}
-		if (RB_IS_PLAYLIST_SOURCE (source) == FALSE) {
-			g_object_unref (source);
-			continue;
-		}
-		g_object_get (source, "name", &source_name, NULL);
-		if (strcmp (name, source_name) == 0)
-			playlist = source;
-
-		g_free (source_name);
-		g_object_unref (source);
-	} while (gtk_tree_model_iter_next (model, &iter) && playlist == NULL);
-
-	return playlist;
+	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)find_playlist_by_name_cb, &d);
+	return d.source;
 }
 
 /**
@@ -1654,7 +1712,7 @@ _get_playlist_by_name (RBPlaylistManager *mgr,
  * Will fail if a playlist with that name already exists.
  * This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_create_static_playlist (RBPlaylistManager *mgr,
@@ -1683,7 +1741,7 @@ rb_playlist_manager_create_static_playlist (RBPlaylistManager *mgr,
  * Deletes the specified playlist.  Will fail if no playlist with
  * that name exists. This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  */
 gboolean
 rb_playlist_manager_delete_playlist (RBPlaylistManager *mgr,
@@ -1715,7 +1773,7 @@ rb_playlist_manager_delete_playlist (RBPlaylistManager *mgr,
  * Fails if no playlist with that name exists.
  * This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_add_to_playlist (RBPlaylistManager *mgr,
@@ -1755,7 +1813,7 @@ rb_playlist_manager_add_to_playlist (RBPlaylistManager *mgr,
  * Fails if no playlist with that name exists.
  * This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_remove_from_playlist (RBPlaylistManager *mgr,
@@ -1797,7 +1855,7 @@ rb_playlist_manager_remove_from_playlist (RBPlaylistManager *mgr,
  * Saves the specified playlist to a file in either M3U or PLS format.
  * This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_export_playlist (RBPlaylistManager *mgr,
