@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -46,16 +54,19 @@ typedef enum
 
 GQuark rb_playlist_manager_error_quark (void);
 
+typedef struct _RBPlaylistManager RBPlaylistManager;
+typedef struct _RBPlaylistManagerClass RBPlaylistManagerClass;
+
 typedef struct RBPlaylistManagerPrivate RBPlaylistManagerPrivate;
 
-typedef struct
+struct _RBPlaylistManager
 {
 	GObject parent;
 
 	RBPlaylistManagerPrivate *priv;
-} RBPlaylistManager;
+};
 
-typedef struct
+struct _RBPlaylistManagerClass
 {
 	GObjectClass parent_class;
 
@@ -64,7 +75,7 @@ typedef struct
 	void	(*playlist_created) (RBPlaylistManager *manager, RBSource *source);
 	void	(*load_start) (RBPlaylistManager *manager);
 	void	(*load_finish) (RBPlaylistManager *manager);
-} RBPlaylistManagerClass;
+};
 
 typedef enum
 {
@@ -95,27 +106,27 @@ RBSource *		rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
 RBSource *		rb_playlist_manager_new_playlist_from_selection_data (RBPlaylistManager *mgr,
                                                                               GtkSelectionData *data);
 
-GList *			rb_playlist_manager_get_playlists (RBPlaylistManager *manager);
+GList *			rb_playlist_manager_get_playlists (RBPlaylistManager *mgr);
 
-gboolean		rb_playlist_manager_get_playlist_names (RBPlaylistManager *manager,
+gboolean		rb_playlist_manager_get_playlist_names (RBPlaylistManager *mgr,
 								gchar ***playlists,
 								GError **error);
-gboolean		rb_playlist_manager_create_static_playlist (RBPlaylistManager *manager,
+gboolean		rb_playlist_manager_create_static_playlist (RBPlaylistManager *mgr,
 								    const gchar *name,
 								    GError **error);
-gboolean		rb_playlist_manager_delete_playlist (RBPlaylistManager *manager,
+gboolean		rb_playlist_manager_delete_playlist (RBPlaylistManager *mgr,
 							     const gchar *name,
 							     GError **error);
-gboolean		rb_playlist_manager_add_to_playlist (RBPlaylistManager *manager,
-							     const gchar *playlist,
+gboolean		rb_playlist_manager_add_to_playlist (RBPlaylistManager *mgr,
+							     const gchar *name,
 							     const gchar *uri,
 							     GError **error);
-gboolean		rb_playlist_manager_remove_from_playlist (RBPlaylistManager *manager,
-								  const gchar *playlist,
+gboolean		rb_playlist_manager_remove_from_playlist (RBPlaylistManager *mgr,
+								  const gchar *name,
 								  const gchar *uri,
 								  GError **error);
-gboolean		rb_playlist_manager_export_playlist (RBPlaylistManager *manager,
-							     const gchar *playlist,
+gboolean		rb_playlist_manager_export_playlist (RBPlaylistManager *mgr,
+							     const gchar *name,
 							     const gchar *uri,
 							     gboolean m3u_format,
 							     GError **error);
