@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -21,6 +29,25 @@
  *
  */
 
+/**
+ * SECTION:rb-shell-preferences
+ * @short_description: preferences dialog
+ *
+ * The preferences dialog is built around a #GtkNotebook widget, with two built-in
+ * pages and additional pages for various sources.
+ *
+ * The 'general' preferences page controls the set of browser views that are visible
+ * (artist and album; genre and artist; or genre, artist, and album), the columns
+ * that are visible, and the appearance of buttons in the main toolbar.  The browser
+ * and column settings apply to all sources.
+ *
+ * The 'playback' preferences page controls whether the crossfading player backend is used,
+ * and if enabled, the crossfade duration and network buffer size.
+ *
+ * Currently, the library and podcast sources add pages to the notebook, for configuring the
+ * location and layout of the library and the podcast download location and update frequency.
+ */
+
 #include <config.h>
 
 #include <string.h>
@@ -60,6 +87,17 @@ void rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
 						      RBShellPreferences *shell_preferences);
 static void rb_shell_preferences_toolbar_style_cb (GtkComboBox *box,
 						   RBShellPreferences *preferences);
+#ifdef HAVE_GSTREAMER_0_10_XFADE
+static void rb_shell_preferences_player_backend_cb (GtkToggleButton *button,
+						    RBShellPreferences *preferences);
+static void rb_shell_preferences_album_crossfade_cb (GtkToggleButton *button,
+						     RBShellPreferences *preferences);
+static void rb_shell_preferences_transition_duration_cb (GtkRange *range,
+							 RBShellPreferences *preferences);
+static void rb_shell_preferences_network_buffer_size_cb (GtkRange *range,
+							 RBShellPreferences *preferences);
+static void update_playback_prefs_sensitivity (RBShellPreferences *preferences);
+#endif
 
 enum
 {
@@ -85,6 +123,13 @@ struct RBShellPreferencesPrivate
 	GtkWidget *quality_check;
 	GtkWidget *year_check;
 
+#ifdef HAVE_GSTREAMER_0_10_XFADE
+	GtkWidget *xfade_backend_check;
+	GtkWidget *album_crossfade_check;
+	GtkWidget *transition_duration;
+	GtkWidget *network_buffer_size;
+#endif
+
 	GSList *browser_views_group;
 
 	GtkWidget *toolbar_style_menu;
@@ -131,11 +176,11 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 
 	shell_preferences->priv = RB_SHELL_PREFERENCES_GET_PRIVATE (shell_preferences);
 
-	g_signal_connect_object (G_OBJECT (shell_preferences),
+	g_signal_connect_object (shell_preferences,
 				 "delete_event",
 				 G_CALLBACK (rb_shell_preferences_window_delete_cb),
 				 shell_preferences, 0);
-	g_signal_connect_object (G_OBJECT (shell_preferences),
+	g_signal_connect_object (shell_preferences,
 				 "response",
 				 G_CALLBACK (rb_shell_preferences_response_cb),
 				 shell_preferences, 0);
@@ -146,7 +191,7 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 	tmp = gtk_dialog_add_button (GTK_DIALOG (shell_preferences),
 			              GTK_STOCK_HELP,
 			              GTK_RESPONSE_HELP);
-	g_signal_connect_object (G_OBJECT (tmp), "clicked",
+	g_signal_connect_object (tmp, "clicked",
 				 G_CALLBACK (help_cb), shell_preferences, 0);
 	gtk_dialog_set_default_response (GTK_DIALOG (shell_preferences),
 					 GTK_RESPONSE_CLOSE);
@@ -220,7 +265,58 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 	eel_gconf_notification_add (CONF_UI_DIR,
 				    (GConfClientNotifyFunc) rb_shell_preferences_ui_pref_changed,
 				    shell_preferences);
-	g_object_unref (G_OBJECT (xml));
+	g_object_unref (xml);
+
+#ifdef HAVE_GSTREAMER_0_10_XFADE
+	/* playback preferences */
+	xml = rb_glade_xml_new ("playback-prefs.glade",
+				"playback_prefs_box",
+				shell_preferences);
+
+	rb_glade_boldify_label (xml, "backend_label");
+	rb_glade_boldify_label (xml, "crossfade_type_label");
+	rb_glade_boldify_label (xml, "duration_label");
+	rb_glade_boldify_label (xml, "buffer_size_label");
+
+	shell_preferences->priv->xfade_backend_check =
+		glade_xml_get_widget (xml, "use_xfade_backend");
+	shell_preferences->priv->album_crossfade_check =
+		glade_xml_get_widget (xml, "album_check");
+	shell_preferences->priv->transition_duration =
+		glade_xml_get_widget (xml, "duration");
+	shell_preferences->priv->network_buffer_size =
+		glade_xml_get_widget (xml, "network_buffer_size");
+
+	g_signal_connect_object (shell_preferences->priv->xfade_backend_check,
+				 "toggled",
+				 G_CALLBACK (rb_shell_preferences_player_backend_cb),
+				 shell_preferences, 0);
+
+	g_signal_connect_object (shell_preferences->priv->album_crossfade_check,
+				 "toggled",
+				 G_CALLBACK (rb_shell_preferences_album_crossfade_cb),
+				 shell_preferences, 0);
+
+	g_signal_connect_object (shell_preferences->priv->transition_duration,
+				 "value-changed",
+				 G_CALLBACK (rb_shell_preferences_transition_duration_cb),
+				 shell_preferences, 0);
+
+	g_signal_connect_object (shell_preferences->priv->network_buffer_size,
+				 "value-changed",
+				 G_CALLBACK (rb_shell_preferences_network_buffer_size_cb),
+				 shell_preferences, 0);
+
+	gtk_notebook_append_page (GTK_NOTEBOOK (shell_preferences->priv->notebook),
+				  glade_xml_get_widget (xml, "playback_prefs_box"),
+				  gtk_label_new (_("Playback")));
+	g_object_unref (xml);
+#endif
+
+	eel_gconf_notification_add (CONF_PLAYER_DIR,
+				    (GConfClientNotifyFunc) rb_shell_preferences_ui_pref_changed,
+				    shell_preferences);
+
 	rb_shell_preferences_sync (shell_preferences);
 }
 
@@ -239,6 +335,14 @@ rb_shell_preferences_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_shell_preferences_parent_class)->finalize (object);
 }
 
+/**
+ * rb_shell_preferences_append_page:
+ * @prefs: the #RBShellPreferences instance
+ * @name: name of the page to append
+ * @widget: the #GtkWidget to use as the contents of the page
+ *
+ * Appends a new page to the preferences dialog notebook.
+ */
 void
 rb_shell_preferences_append_page (RBShellPreferences *prefs,
 				  const char *name,
@@ -269,6 +373,15 @@ rb_shell_preferences_append_view_page (RBShellPreferences *prefs,
 	rb_shell_preferences_append_page (prefs, name, widget);
 }
 
+/**
+ * rb_shell_preferences_new:
+ * @views: list of #RBSource objects to check for preferences pages
+ *
+ * Creates the #RBShellPreferences instance, populating it with the
+ * preferences pages for the sources in the list.
+ *
+ * Return value: the #RBShellPreferences instance
+ */
 GtkWidget *
 rb_shell_preferences_new (GList *views)
 {
@@ -283,7 +396,12 @@ rb_shell_preferences_new (GList *views)
 	{
 		char *name = NULL;
 		g_object_get (views->data, "name", &name, NULL);
-		g_assert (name != NULL);
+		if (name == NULL) {
+			g_warning ("Source %p of type %s has no name",
+				   views->data,
+				   G_OBJECT_TYPE_NAME (views->data));
+			continue;
+		}
 		rb_shell_preferences_append_view_page (shell_preferences,
 						       name,
 						       RB_SOURCE (views->data));
@@ -324,6 +442,14 @@ rb_shell_preferences_ui_pref_changed (GConfClient *client,
 	rb_shell_preferences_sync (shell_preferences);
 }
 
+/**
+ * rb_shell_preferences_column_check_changed_cb:
+ * @butt: the #GtkCheckButton that was changed
+ * @shell_preferences: the #RBShellPreferences instance
+ *
+ * Signal handler used for the checkboxes used to configure the set of visible columns.
+ * This updates the GConf key that contains the list of visible columns.
+ */
 void
 rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 					      RBShellPreferences *shell_preferences)
@@ -468,6 +594,29 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 					   G_CALLBACK (rb_shell_preferences_toolbar_style_cb),
 					   shell_preferences);
 
+#ifdef HAVE_GSTREAMER_0_10_XFADE
+	{
+		gboolean b;
+		float duration;
+		int buffer_size;
+
+		/* player preferences */
+		b = eel_gconf_get_boolean (CONF_PLAYER_USE_XFADE_BACKEND);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->xfade_backend_check), b);
+
+		b = eel_gconf_get_boolean (CONF_PLAYER_TRANSITION_ALBUM_CHECK);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->album_crossfade_check), !b);
+
+		duration = eel_gconf_get_float (CONF_PLAYER_TRANSITION_TIME);
+		gtk_range_set_value (GTK_RANGE (shell_preferences->priv->transition_duration), duration);
+
+		buffer_size = eel_gconf_get_integer (CONF_PLAYER_NETWORK_BUFFER_SIZE);
+		gtk_range_set_value (GTK_RANGE (shell_preferences->priv->network_buffer_size), buffer_size);
+
+		update_playback_prefs_sensitivity (shell_preferences);
+	}
+#endif
+
 	shell_preferences->priv->loading = FALSE;
 }
 
@@ -485,6 +634,14 @@ rb_shell_preferences_toolbar_style_cb (GtkComboBox *box, RBShellPreferences *pre
 	eel_gconf_set_integer (CONF_UI_TOOLBAR_STYLE, selection);
 }
 
+/**
+ * rb_shell_preferences_browser_views_activated_cb:
+ * @widget: the radio button that was selected
+ * @shell_preferences: the #RBShellPreferences instance
+ *
+ * Signal handler used for the radio buttons used to configure the
+ * visible browser views.
+ */
 void
 rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
 						 RBShellPreferences *shell_preferences)
@@ -498,3 +655,60 @@ rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
 
 	eel_gconf_set_integer (CONF_UI_BROWSER_VIEWS, index);
 }
+
+#ifdef HAVE_GSTREAMER_0_10_XFADE
+
+static void
+update_playback_prefs_sensitivity (RBShellPreferences *preferences)
+{
+	gboolean backend;
+
+	backend = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (preferences->priv->xfade_backend_check));
+
+	gtk_widget_set_sensitive (preferences->priv->album_crossfade_check, backend);
+	gtk_widget_set_sensitive (preferences->priv->network_buffer_size, backend);
+	gtk_widget_set_sensitive (preferences->priv->transition_duration, backend);
+}
+
+static void
+rb_shell_preferences_player_backend_cb (GtkToggleButton *button,
+					RBShellPreferences *preferences)
+{
+	update_playback_prefs_sensitivity (preferences);
+
+	eel_gconf_set_boolean (CONF_PLAYER_USE_XFADE_BACKEND,
+			       gtk_toggle_button_get_active (button));
+}
+
+static void
+rb_shell_preferences_album_crossfade_cb (GtkToggleButton *button,
+					 RBShellPreferences *preferences)
+{
+	/* sense is inverted here because the wording sounded more natural
+	 * this way.  "[ ] Don't do x" sucks.
+	 */
+	eel_gconf_set_boolean (CONF_PLAYER_TRANSITION_ALBUM_CHECK,
+			       !gtk_toggle_button_get_active (button));
+}
+
+static void
+rb_shell_preferences_transition_duration_cb (GtkRange *range,
+					     RBShellPreferences *preferences)
+{
+	gdouble v;
+
+	v = gtk_range_get_value (range);
+	eel_gconf_set_float (CONF_PLAYER_TRANSITION_TIME, (float)v);
+}
+
+static void
+rb_shell_preferences_network_buffer_size_cb (GtkRange *range,
+					     RBShellPreferences *preferences)
+{
+	gdouble v;
+
+	v = gtk_range_get_value (range);
+	eel_gconf_set_integer (CONF_PLAYER_NETWORK_BUFFER_SIZE, (int)v);
+}
+
+#endif
