@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -38,6 +46,20 @@
 #include "eel-gconf-extensions.h"
 #include "rb-util.h"
 
+/**
+ * SECTION:rb-source-header
+ * @short_description: container for search box and toolbar
+ *
+ * The source header contains the search box and the search
+ * action toolbar.  It remembers the search text and browser
+ * visibility state for each source, applying the stored state
+ * for the newly selected source when the source selection changes.
+ *
+ * The search bar contains a set of actions determined by the
+ * selected source.  The source is responsible for tracking
+ * which search action is active.
+ */
+
 static void rb_source_header_class_init (RBSourceHeaderClass *klass);
 static void rb_source_header_init (RBSourceHeader *shell_player);
 static void rb_source_header_finalize (GObject *object);
@@ -193,6 +215,11 @@ rb_source_header_class_init (RBSourceHeaderClass *klass)
 	object_class->set_property = rb_source_header_set_property;
 	object_class->get_property = rb_source_header_get_property;
 
+	/**
+	 * RBSourceHeader:source:
+	 *
+	 * The currently selected #RBSource.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SOURCE,
 					 g_param_spec_object ("source",
@@ -200,6 +227,11 @@ rb_source_header_class_init (RBSourceHeaderClass *klass)
 							      "RBSource object",
 							      RB_TYPE_SOURCE,
 							      G_PARAM_READWRITE));
+	/**
+	 * RBSourceHeader:action-group:
+	 *
+	 * The #GtkActionGroup to add actions to.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_ACTION_GROUP,
 					 g_param_spec_object ("action-group",
@@ -207,6 +239,11 @@ rb_source_header_class_init (RBSourceHeaderClass *klass)
 							      "GtkActionGroup object",
 							      GTK_TYPE_ACTION_GROUP,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBSourceHeader:ui-manager:
+	 *
+	 * The #GtkUIManager instance.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_UI_MANAGER,
 					 g_param_spec_object ("ui-manager",
@@ -229,7 +266,6 @@ rb_source_header_init (RBSourceHeader *header)
 	header->priv->tooltips = gtk_tooltips_new ();
 	gtk_tooltips_enable (header->priv->tooltips);
 
-	gtk_table_set_homogeneous (GTK_TABLE (header), TRUE);
 	gtk_table_set_col_spacings (GTK_TABLE (header), 5);
 	gtk_table_resize (GTK_TABLE (header), 1, 3);
 
@@ -359,12 +395,15 @@ rb_source_header_set_source_internal (RBSourceHeader *header,
 			gtk_widget_hide (GTK_WIDGET (header));
 		else
 			gtk_widget_show (GTK_WIDGET (header));
-	}
 
-	/* merge the source-specific UI */
-	actions = rb_source_get_search_actions (source);
-	g_list_foreach (actions, (GFunc)merge_source_ui_cb, header);
-	rb_list_deep_free (actions);
+		/* merge the source-specific UI */
+		actions = rb_source_get_search_actions (source);
+		g_list_foreach (actions, (GFunc)merge_source_ui_cb, header);
+		rb_list_deep_free (actions);
+	} else {
+		/* no selected source -> hide source header */
+		gtk_widget_hide (GTK_WIDGET (header));
+	}
 
 	rb_source_header_sync_control_state (header);
 }
@@ -422,18 +461,34 @@ rb_source_header_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_source_header_set_source:
+ * @header: the #RBSourceHeader
+ * @source: the new selected #RBSource
+ *
+ * Updates the source header when a new source is selected.
+ */
 void
 rb_source_header_set_source (RBSourceHeader *header,
 			     RBSource *source)
 {
 	g_return_if_fail (RB_IS_SOURCE_HEADER (header));
-	g_return_if_fail (RB_IS_SOURCE (source));
+	g_return_if_fail (source == NULL || RB_IS_SOURCE (source));
 
 	g_object_set (G_OBJECT (header),
 		      "source", source,
 		      NULL);
 }
 
+/**
+ * rb_source_header_new:
+ * @mgr: the #GtkUIManager
+ * @actiongroup: the #GtkActionGroup to add actions to
+ *
+ * Creates the #RBSourceHeader.
+ *
+ * Return value: the #RBSourceHeader instance
+ */
 RBSourceHeader *
 rb_source_header_new (GtkUIManager   *mgr,
 		      GtkActionGroup *actiongroup)
@@ -515,6 +570,13 @@ rb_source_header_search_cb (RBSearchEntry *search,
 	rb_source_header_sync_control_state (header);
 }
 
+/**
+ * rb_source_header_clear_search:
+ * @header: the #RBSourceHeader
+ *
+ * Clears the search box and resets all search state for the
+ * current selected source.
+ */
 void
 rb_source_header_clear_search (RBSourceHeader *header)
 {
@@ -551,6 +613,13 @@ rb_source_header_view_browser_changed_cb (GtkAction *action,
 	rb_source_header_sync_control_state (header);
 }
 
+/**
+ * rb_source_header_sync_control_state:
+ * @header: the #RBSourceHeader
+ *
+ * Updates the sensitivity and active state of various actions
+ * related to the source header.
+ */
 void
 rb_source_header_sync_control_state (RBSourceHeader *header)
 {
@@ -586,6 +655,12 @@ rb_source_header_search_activate_cb (RBSearchEntry *search,
 	gtk_widget_grab_focus (GTK_WIDGET (header->priv->selected_source));
 }
 
+/**
+ * rb_source_header_focus_search_box:
+ * @header: the #RBSourceHeader
+ *
+ * Grabs input focus for the search box.
+ */
 void
 rb_source_header_focus_search_box (RBSourceHeader *header)
 {
