@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -21,6 +29,18 @@
  *
  */
 
+/**
+ * SECTION:rb-browser-source
+ * @short_description: base class for sources that include genre/artist/album browsers
+ *
+ * This class simplifies implementation of sources that include genre/artist/album browsers.
+ * It also handles searching (using the search box) and a few other UI niceties.
+ * 
+ * Instances of browser sources will use a query that will match all entries of
+ * the entry type assigned to the source, so it's mostly suited for sources that
+ * have an entry type of their own.
+ */
+
 #include "config.h"
 
 #include <string.h>
@@ -90,7 +110,6 @@ static void impl_delete (RBSource *source);
 static void impl_search (RBSource *source, const char *text);
 static void impl_reset_filters (RBSource *source);
 static void impl_song_properties (RBSource *source);
-static gboolean impl_show_popup (RBSource *source);
 static GList *impl_get_search_actions (RBSource *source);
 static void impl_browser_toggled (RBSource *asource, gboolean disclosed);
 static void default_show_entry_popup (RBBrowserSource *source);
@@ -165,7 +184,8 @@ static const GtkTargetEntry songs_view_drag_types[] = {
 enum
 {
 	PROP_0,
-	PROP_SORTING_KEY
+	PROP_SORTING_KEY,
+	PROP_BASE_QUERY_MODEL
 };
 
 G_DEFINE_ABSTRACT_TYPE (RBBrowserSource, rb_browser_source, RB_TYPE_SOURCE)
@@ -197,7 +217,6 @@ rb_browser_source_class_init (RBBrowserSourceClass *klass)
 	source_class->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_get_search_actions = impl_get_search_actions;
 	source_class->impl_browser_toggled = impl_browser_toggled;
 
@@ -214,6 +233,10 @@ rb_browser_source_class_init (RBBrowserSourceClass *klass)
 							      NULL,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_override_property (object_class,
+					  PROP_BASE_QUERY_MODEL,
+					  "base-query-model");
+
 	g_type_class_add_private (klass, sizeof (RBBrowserSourcePrivate));
 }
 
@@ -528,6 +551,9 @@ rb_browser_source_get_property (GObject *object,
 	case PROP_SORTING_KEY:
 		g_value_set_string (value, source->priv->sorting_key);
 		break;
+	case PROP_BASE_QUERY_MODEL:
+		g_value_set_object (value, source->priv->cached_all_query);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -787,13 +813,15 @@ impl_browser_toggled (RBSource *asource, gboolean disclosed)
 	}
 }
 
-static gboolean
-impl_show_popup (RBSource *source)
-{
-	_rb_source_show_popup (RB_SOURCE (source), "/BrowserSourcePopup");
-	return TRUE;
-}
-
+/**
+ * rb_browser_source_get_paned_key:
+ * @source: a #RBBrowserSource
+ *
+ * Retrieves the GConf key that stores the height of the browser pane for the source.
+ * This is a virtual method that should be implemented by subclasses.
+ *
+ * Return value: allocated string containing the GConf key name
+ */
 char *
 rb_browser_source_get_paned_key (RBBrowserSource *source)
 {
@@ -805,6 +833,15 @@ rb_browser_source_get_paned_key (RBBrowserSource *source)
 		return NULL;
 }
 
+/**
+ * rb_browser_source_has_drop_support:
+ * @source: a #RBBrowser
+ *
+ * This is a virtual method that should be implemented by subclasses.  It returns %TRUE
+ * if drag and drop target support for the source should be activated.
+ *
+ * Return value: %TRUE if drop support should be activated
+ */
 gboolean
 rb_browser_source_has_drop_support (RBBrowserSource *source)
 {
