@@ -7,6 +7,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -18,10 +26,6 @@
  *
  */
 
-/*
- * This source lists files rhythmbox failed to import.
- */
-
 #include <config.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -44,7 +48,7 @@ static void rb_import_errors_source_songs_show_popup_cb (RBEntryView *view,
 							 gboolean over_entry,
 							 RBImportErrorsSource *source);
 
-struct RBImportErrorsSourcePrivate
+struct _RBImportErrorsSourcePrivate
 {
 	RhythmDB *db;
 	RBEntryView *view;
@@ -52,6 +56,29 @@ struct RBImportErrorsSourcePrivate
 
 G_DEFINE_TYPE (RBImportErrorsSource, rb_import_errors_source, RB_TYPE_SOURCE);
 
+/**
+ * SECTION:rb-import-errors-source
+ * @short_description: source for displaying import errors
+ *
+ * This source is used to display the names of files that could not
+ * be imported into the library, along with any error messages from
+ * the import process.  When there are no import errors to display,
+ * the source is hidden.
+ *
+ * The source allows the user to delete the import error entries,
+ * and to move the files to the trash.
+ *
+ * When a file import fails, a #RhythmDBEntry is created with a
+ * specific entry type for import errors.  This source uses a query
+ * model that matches all such import error entries.
+ *
+ * To keep import errors from removable devices separate from those
+ * from the main library, multiple import error sources can be created,
+ * with separate entry types.  The generic audio player plugin, for
+ * example, creates an import error source for each device and inserts
+ * it into the source list as a child of the main source for the device.
+ */
+
 static void
 rb_import_errors_source_class_init (RBImportErrorsSourceClass *klass)
 {
@@ -90,7 +117,7 @@ rb_import_errors_source_init (RBImportErrorsSource *source)
 
 	source->priv = G_TYPE_INSTANCE_GET_PRIVATE (source, RB_TYPE_IMPORT_ERRORS_SOURCE, RBImportErrorsSourcePrivate);
 
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 					   GTK_STOCK_DIALOG_ERROR,
 					   size,
@@ -181,9 +208,19 @@ impl_get_entry_view (RBSource *asource)
 	return source->priv->view;
 }
 
+/**
+ * rb_import_errors_source_new:
+ * @shell: the #RBShell instance
+ * @entry_type: the entry type to display in the source
+ *
+ * Creates a new source for displaying import errors of the
+ * specified type.
+ *
+ * Return value: a new import error source
+ */
 RBSource *
 rb_import_errors_source_new (RBShell *shell,
-			     RBLibrarySource *library)
+			     RhythmDBEntryType entry_type)
 {
 	RBSource *source;
 
@@ -192,7 +229,8 @@ rb_import_errors_source_new (RBShell *shell,
 					  "shell", shell,
 					  "visibility", FALSE,
 					  "hidden-when-empty", TRUE,
-					  "entry-type", RHYTHMDB_ENTRY_TYPE_IMPORT_ERROR,
+					  "source-group", RB_SOURCE_GROUP_LIBRARY,
+					  "entry-type", entry_type,
 					  NULL));
 	return source;
 }
