@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -21,6 +29,23 @@
  *
  */
 
+/**
+ * SECTION:rb-library-source
+ * @short_description: main library source, containing all local songs
+ *
+ * The library source contains all local songs that have been imported
+ * into the database.
+ *
+ * It provides a preferences page for configuring the library location,
+ * the directory structure to use when transferring new files into
+ * the library from another source, and the preferred audio encoding
+ * to use.
+ *
+ * If multiple library locations are set in GConf, the library source
+ * creates a child source for each location, which will only show
+ * files found under that location.
+ */
+
 #include "config.h"
 
 #include <string.h>
@@ -294,8 +319,13 @@ rb_library_source_constructor (GType type,
 	/* Set up the default library location if there's no library location set */
 	list = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
 	if (g_slist_length (list) == 0) {
-		list = g_slist_prepend (list, g_strdup (rb_music_dir ()));
-		eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
+		char *music_dir_uri;
+
+		music_dir_uri = g_filename_to_uri (rb_music_dir (), NULL, NULL);
+		if (music_dir_uri != NULL) {
+			list = g_slist_prepend (list, music_dir_uri);
+			eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
+		}
 	}
 	rb_slist_deep_free (list);
 
@@ -320,6 +350,14 @@ rb_library_source_constructor (GType type,
 	return G_OBJECT (source);
 }
 
+/**
+ * rb_library_source_new:
+ * @shell: the #RBShell
+ *
+ * Creates and returns the #RBLibrarySource instance
+ *
+ * Return value: the #RBLibrarySource
+ */
 RBSource *
 rb_library_source_new (RBShell *shell)
 {
@@ -641,75 +679,6 @@ impl_get_paned_key (RBBrowserSource *status)
 	return g_strdup (CONF_STATE_PANED_POSITION);
 }
 
-static void
-rb_library_source_add_location_entry_changed_cb (GtkEntry *entry,
-						 GtkWidget *target)
-{
-	gtk_widget_set_sensitive (target, g_utf8_strlen (gtk_entry_get_text (GTK_ENTRY (entry)), -1) > 0);
-}
-
-void
-rb_library_source_add_location (RBLibrarySource *source, GtkWindow *win)
-{
-	GladeXML *xml = rb_glade_xml_new ("uri.glade",
-					  "open_uri_dialog_content",
-					  source);
-	GtkWidget *content, *uri_widget, *open_button;
-	GtkWidget *dialog = gtk_dialog_new_with_buttons (_("Add Location"),
-							 win,
-							 GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
-							 GTK_STOCK_CANCEL,
-							 GTK_RESPONSE_CANCEL,
-							 NULL);
-
-	g_return_if_fail (dialog != NULL);
-
-	open_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
-					     GTK_STOCK_OPEN,
-					     GTK_RESPONSE_OK);
-	gtk_widget_set_sensitive (open_button, FALSE);
-
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-					 GTK_RESPONSE_OK);
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
-
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-
-	content = glade_xml_get_widget (xml, "open_uri_dialog_content");
-
-	g_return_if_fail (content != NULL);
-
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-			    content, FALSE, FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (content), 5);
-
-	uri_widget = glade_xml_get_widget (xml, "uri");
-
-	g_return_if_fail (uri_widget != NULL);
-
-	g_signal_connect_object (G_OBJECT (uri_widget),
-				 "changed",
-				 G_CALLBACK (rb_library_source_add_location_entry_changed_cb),
-				 open_button, 0);
-
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
-		char *uri = gtk_editable_get_chars (GTK_EDITABLE (uri_widget), 0, -1);
-		if (uri != NULL) {
-			GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri);
-			if (vfsuri != NULL) {
-				rhythmdb_add_uri (source->priv->db, uri);
-				gnome_vfs_uri_unref (vfsuri);
-			} else {
-				rb_debug ("invalid uri: \"%s\"", uri);
-			}
-
-		}
-	}
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-
-}
-
 static gboolean
 impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 {
@@ -839,7 +808,7 @@ sanitize_pattern (const char *pat)
 	}
 }
 
-/**
+/*
  * Parse a filename pattern and replace markers with values from a TrackDetails
  * structure.
  *
@@ -1230,7 +1199,7 @@ impl_can_paste (RBSource *asource)
 
 #ifdef ENABLE_TRACK_TRANSFER
 static void
-completed_cb (RhythmDBEntry *entry, const char *dest, RBLibrarySource *source)
+completed_cb (RhythmDBEntry *entry, const char *dest, guint64 dest_size, RBLibrarySource *source)
 {
 	rhythmdb_add_uri (source->priv->db, dest);
 }
