@@ -7,6 +7,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -18,7 +26,7 @@
  *
  */
 
-/* 
+/*
  * Header for play queue source class
  */
 
@@ -36,15 +44,18 @@ G_BEGIN_DECLS
 #define RB_IS_PLAY_QUEUE_SOURCE_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_PLAY_QUEUE_SOURCE))
 #define RB_PLAY_QUEUE_SOURCE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_PLAY_QUEUE_SOURCE, RBPlayQueueSourceClass))
 
-typedef struct
+typedef struct _RBPlayQueueSource RBPlayQueueSource;
+typedef struct _RBPlayQueueSourceClass RBPlayQueueSourceClass;
+
+struct _RBPlayQueueSource
 {
 	RBStaticPlaylistSource parent;
-} RBPlayQueueSource;
+};
 
-typedef struct
+struct _RBPlayQueueSourceClass
 {
 	RBStaticPlaylistSourceClass parent;
-} RBPlayQueueSourceClass;
+};
 
 GType		rb_play_queue_source_get_type 		(void);
 
@@ -57,4 +68,3 @@ void		rb_play_queue_source_clear_queue	(RBPlayQueueSource *source);
 G_END_DECLS
 
 #endif /* __RB_PLAY_QUEUE_SOURCE_H */
-
