@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -40,6 +48,14 @@ static GObject *rb_static_playlist_source_constructor (GType type, guint n_const
 						       GObjectConstructParam *construct_properties);
 static void rb_static_playlist_source_dispose (GObject *object);
 static void rb_static_playlist_source_finalize (GObject *object);
+static void rb_static_playlist_source_set_property (GObject *object,
+						    guint prop_id,
+						    const GValue *value,
+						    GParamSpec *pspec);
+static void rb_static_playlist_source_get_property (GObject *object,
+						    guint prop_id,
+						    GValue *value,
+						    GParamSpec *pspec);
 
 /* source methods */
 static GList * impl_cut (RBSource *source);
@@ -76,6 +92,9 @@ static void rb_static_playlist_source_row_inserted (GtkTreeModel *model,
 						    GtkTreePath *path,
 						    GtkTreeIter *iter,
 						    RBStaticPlaylistSource *source);
+static gboolean rb_static_playlist_source_filter_entry_drop (RhythmDBQueryModel *model,
+							     RhythmDBEntry *entry,
+							     RBStaticPlaylistSource *source);
 static void rb_static_playlist_source_non_entry_dropped (GtkTreeModel *model,
 							 const char *uri,
 							 int position,
@@ -98,6 +117,12 @@ static GtkRadioActionEntry rb_static_playlist_source_radio_actions [] =
 	{ "StaticPlaylistSearchTitles", NULL, N_("Titles"), NULL, N_("Search titles"), 3 }
 };
 
+enum
+{
+	PROP_0,
+	PROP_BASE_QUERY_MODEL
+};
+
 G_DEFINE_TYPE (RBStaticPlaylistSource, rb_static_playlist_source, RB_TYPE_PLAYLIST_SOURCE)
 #define RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
 								RB_TYPE_STATIC_PLAYLIST_SOURCE, \
@@ -131,6 +156,8 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 	object_class->constructor = rb_static_playlist_source_constructor;
 	object_class->dispose = rb_static_playlist_source_dispose;
 	object_class->finalize = rb_static_playlist_source_finalize;
+	object_class->set_property = rb_static_playlist_source_set_property;
+	object_class->get_property = rb_static_playlist_source_get_property;
 
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
@@ -150,6 +177,10 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 
 	playlist_class->impl_save_contents_to_xml = impl_save_contents_to_xml;
 
+	g_object_class_override_property (object_class,
+					  PROP_BASE_QUERY_MODEL,
+					  "base-query-model");
+
 	g_type_class_add_private (klass, sizeof (RBStaticPlaylistSourcePrivate));
 }
 
@@ -239,6 +270,10 @@ rb_static_playlist_source_constructor (GType type,
 	priv->base_model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (psource));
 	g_object_set (priv->base_model, "show-hidden", TRUE, NULL);
 	g_object_ref (priv->base_model);
+	g_signal_connect_object (priv->base_model,
+				 "filter-entry-drop",
+				 G_CALLBACK (rb_static_playlist_source_filter_entry_drop),
+				 source, 0);
 
 	priv->paned = gtk_vpaned_new ();
 
@@ -297,13 +332,17 @@ rb_static_playlist_source_constructor (GType type,
 }
 
 RBSource *
-rb_static_playlist_source_new (RBShell *shell, const char *name, gboolean local, RhythmDBEntryType entry_type)
+rb_static_playlist_source_new (RBShell *shell, const char *name, const char *sorting_name, gboolean local, RhythmDBEntryType entry_type)
 {
 	if (name == NULL)
 		name = "";
 
+	if (sorting_name == NULL)
+		sorting_name = "";
+
 	return RB_SOURCE (g_object_new (RB_TYPE_STATIC_PLAYLIST_SOURCE,
 					"name", name,
+					"sorting-name", sorting_name,
 					"shell", shell,
 					"is-local", local,
 					"entry-type", entry_type,
@@ -311,6 +350,39 @@ rb_static_playlist_source_new (RBShell *shell, const char *name, gboolean local,
 					NULL));
 }
 
+static void
+rb_static_playlist_source_set_property (GObject *object,
+					guint prop_id,
+					const GValue *value,
+					GParamSpec *pspec)
+{
+	/*RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (object);*/
+
+	switch (prop_id) {
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+rb_static_playlist_source_get_property (GObject *object,
+					guint prop_id,
+					GValue *value,
+					GParamSpec *pspec)
+{
+	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_BASE_QUERY_MODEL:
+		g_value_set_object (value, priv->base_model);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 void
 rb_static_playlist_source_load_from_xml (RBStaticPlaylistSource *source, xmlNodePtr node)
 {
@@ -336,6 +408,7 @@ RBSource *
 rb_static_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
 {
 	RBSource *psource = rb_static_playlist_source_new (shell,
+							   NULL,
 							   NULL,
 							   TRUE,
 							   RHYTHMDB_ENTRY_TYPE_SONG);
@@ -642,21 +715,12 @@ rb_static_playlist_source_add_location_internal (RBStaticPlaylistSource *source,
 	entry = rhythmdb_entry_lookup_by_location (db, location);
 	if (entry) {
 		RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
-		RhythmDBEntryType entry_type;
 
-		g_object_get (source, "entry-type", &entry_type, NULL);
-		if (entry_type != RHYTHMDB_ENTRY_TYPE_INVALID &&
-		    rhythmdb_entry_get_entry_type (entry) != entry_type) {
-			g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
-			rb_debug ("attempting to add an entry of the wrong type to playlist");
-			return;
+		if (_rb_source_check_entry_type (RB_SOURCE (source), entry)) {
+			rhythmdb_entry_ref (entry);
+			rhythmdb_query_model_add_entry (priv->base_model, entry, index);
+			rhythmdb_entry_unref (entry);
 		}
-
-		rhythmdb_entry_ref (entry);
-		rhythmdb_query_model_add_entry (priv->base_model, entry, index);
-		rhythmdb_entry_unref (entry);
-
-		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	}
 
 	rb_playlist_source_add_to_map (psource, location);
@@ -801,6 +865,18 @@ rb_static_playlist_source_rows_reordered (GtkTreeModel *model,
 	rb_playlist_source_mark_dirty (RB_PLAYLIST_SOURCE (source));
 }
 
+static gboolean
+rb_static_playlist_source_filter_entry_drop (RhythmDBQueryModel *model,
+					     RhythmDBEntry *entry, 
+					     RBStaticPlaylistSource *source)
+{
+	if (_rb_source_check_entry_type (RB_SOURCE (source), entry)) {
+		rb_debug ("allowing drop of entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+		return TRUE;
+	}
+	rb_debug ("preventing drop of entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+	return FALSE;
+}
 
 static GList *
 impl_get_search_actions (RBSource *source)
@@ -884,12 +960,3 @@ impl_want_uri (RBSource *source, const char *uri)
 	return 0;
 }
 
-void
-rb_static_playlist_source_shuffle_playlist(RBStaticPlaylistSource *source) 
-{
-	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
-
-	rhythmdb_query_model_shuffle_entries (priv->base_model);
-
-	rb_playlist_source_mark_dirty (RB_PLAYLIST_SOURCE (source));
-}
