@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -60,7 +68,7 @@ static gboolean rb_rating_button_press_cb (GtkWidget *widget,
 					   GdkEventButton *event,
 					   RBRating *rating);
 
-struct RBRatingPrivate
+struct _RBRatingPrivate
 {
 	double rating;
 	RBRatingPixbufs *pixbufs;
@@ -69,6 +77,14 @@ struct RBRatingPrivate
 G_DEFINE_TYPE (RBRating, rb_rating, GTK_TYPE_EVENT_BOX)
 #define RB_RATING_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_RATING, RBRatingPrivate))
 
+/**
+ * SECTION:rb-rating
+ * @short_description: widget for displaying song ratings
+ *
+ * This widget displays a rating (0-5 stars) and allows the user to
+ * alter the rating by clicking.
+ */
+
 enum
 {
 	PROP_0,
@@ -83,7 +99,6 @@ enum
 
 static guint rb_rating_signals[LAST_SIGNAL] = { 0 };
 
-
 static void
 rb_rating_class_init (RBRatingClass *klass)
 {
@@ -99,8 +114,21 @@ rb_rating_class_init (RBRatingClass *klass)
 	widget_class->expose_event = rb_rating_expose;
 	widget_class->size_request = rb_rating_size_request;
 
+	/**
+	 * RBRating:rating:
+	 *
+	 * The rating displayed in the widget, as a floating point value
+	 * between 0.0 and 5.0.
+	 */
 	rb_rating_install_rating_property (object_class, PROP_RATING);
 
+	/**
+	 * RBRating::rated:
+	 * @rating: the #RBRating
+	 * @score: the new rating
+	 *
+	 * Emitted when the user changes the rating.
+	 */
 	rb_rating_signals[RATED] =
 		g_signal_new ("rated",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -162,7 +190,6 @@ rb_rating_get_property (GObject *object,
 	}
 }
 
-
 static void
 rb_rating_set_property (GObject *object,
 			guint param_id,
@@ -182,6 +209,11 @@ rb_rating_set_property (GObject *object,
 	}
 }
 
+/**
+ * rb_rating_new:
+ *
+ * Return value: a new #RBRating widget.
+ */
 RBRating *
 rb_rating_new ()
 {
@@ -237,7 +269,6 @@ rb_rating_expose (GtkWidget *widget,
 				  widget->allocation.width,
 				  widget->allocation.height);
 
-
 		/* draw the stars */
 		if (rating->priv->pixbufs != NULL) {
 			ret = rb_rating_render_stars (widget,
