@@ -10,6 +10,15 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -56,6 +65,7 @@ static GObject *rb_song_info_constructor (GType type, guint n_construct_properti
 					  GObjectConstructParam *construct_properties);
 
 static void rb_song_info_show (GtkWidget *widget);
+static void rb_song_info_dispose (GObject *object);
 static void rb_song_info_finalize (GObject *object);
 static void rb_song_info_set_property (GObject *object,
 				       guint prop_id,
@@ -79,6 +89,7 @@ static void rb_song_info_update_bitrate (RBSongInfo *song_info);
 static void rb_song_info_update_buttons (RBSongInfo *song_info);
 static void rb_song_info_update_rating (RBSongInfo *song_info);
 static void rb_song_info_update_year (RBSongInfo *song_info);
+static void rb_song_info_update_date_added (RBSongInfo *song_info);
 static void rb_song_info_update_playback_error (RBSongInfo *song_info);
 
 static void rb_song_info_backward_clicked_cb (GtkWidget *button,
@@ -127,6 +138,7 @@ struct RBSongInfoPrivate
 	GtkWidget   *name;
 	GtkWidget   *location;
 	GtkWidget   *filesize;
+	GtkWidget   *date_added;
 	GtkWidget   *play_count;
 	GtkWidget   *last_played;
 	GtkWidget   *rating;
@@ -138,6 +150,35 @@ struct RBSongInfoPrivate
 
 #define RB_SONG_INFO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SONG_INFO, RBSongInfoPrivate))
 
+/**
+ * SECTION:rb-song-info
+ * @short_description: song properties dialog
+ *
+ * Displays song properties and, if we know how to edit tags in the file,
+ * allows the user to edit them.
+ *
+ * This class has two modes.  It can display and edit properties of a single
+ * entry, in which case it uses a #GtkNotebook to split the properties across
+ * 'basic' and 'details' pages, and it can display and edit properties of
+ * multiple entries at a time, in which case a smaller set of properties is
+ * displayed in a single set.
+ *
+ * In single-entry mode, it is possible to add extra pages to the #GtkNotebook
+ * widget in the dialog.  The 'create-song-info' signal is emitted by the #RBShell
+ * object, allowing signal handlers to add pages by calling #rb_song_info_append_page.
+ * The lyrics plugin is currently the only place where this ability is used.
+ * In this mode, the dialog features 'back' and 'forward' buttons that move to the
+ * next or previous entries from the currently displayed track list.
+ *
+ * In multiple-entry mode, only the set of properties that can usefully be set
+ * across multiple entries at once are displayed.
+ *
+ * When the dialog is closed, any changes made will be applied to the entry (or entries)
+ * that were displayed in the dialog.  For songs in the library, this will result
+ * in the song tags being updated on disk.  For other entry types, this only updates
+ * the data store in the database.
+ */
+
 enum
 {
 	PRE_METADATA_CHANGE,
@@ -150,7 +191,8 @@ enum
 	PROP_0,
 	PROP_SOURCE,
 	PROP_ENTRY_VIEW,
-	PROP_CURRENT_ENTRY
+	PROP_CURRENT_ENTRY,
+	PROP_SELECTED_ENTRIES
 };
 
 static guint rb_song_info_signals[LAST_SIGNAL] = { 0 };
@@ -169,6 +211,13 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 
 	widget_class->show = rb_song_info_show;
 
+	/**
+	 * RBSongInfo:source:
+	 *
+	 * The #RBSource that created the song properties window.  Used to update
+	 * for track list changes, and to find the sets of albums, artist, and genres
+	 * to use for tag edit completion.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SOURCE,
 					 g_param_spec_object ("source",
@@ -176,6 +225,13 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 					                      "RBSource object",
 					                      RB_TYPE_SOURCE,
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBSongInfo:entry-view:
+	 *
+	 * The #RBEntryView for the source that created the song properties window.  Used
+	 * find the set of selected entries, and to change the selection when the 'back' and
+	 * 'forward' buttons are pressed.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_VIEW,
 					 g_param_spec_object ("entry-view",
@@ -183,6 +239,12 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 					                      "RBEntryView object",
 					                      RB_TYPE_ENTRY_VIEW,
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBSongInfo:current-entry:
+	 *
+	 * The #RhythmDBEntry that is currently being displayed.  Will be NULL for
+	 * multiple-entry song properties windows.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_CURRENT_ENTRY,
 					 g_param_spec_boxed ("current-entry",
@@ -191,8 +253,36 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 							     RHYTHMDB_TYPE_ENTRY,
 					                     G_PARAM_READABLE));
 
+	/**
+	 * RBSongInfo:selected-entries:
+	 *
+	 * The set of #RhythmDBEntry objects currently being displayed.  Valid for both
+	 * single-entry and multiple-entry song properties windows.
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_SELECTED_ENTRIES,
+					 g_param_spec_value_array ("selected-entries",
+								   "Selected entries",
+								   "List of selected entries, if this is a multiple-entry dialog",
+								   g_param_spec_boxed ("selected-entry",
+										       "Selected entry",
+										       "RhythmDBEntry for a selected entry",
+										       RHYTHMDB_TYPE_ENTRY,
+										       G_PARAM_READABLE),
+								   G_PARAM_READABLE));
+
+	object_class->dispose = rb_song_info_dispose;
 	object_class->finalize = rb_song_info_finalize;
 
+	/**
+	 * RBSongInfo::pre-metadata-change:
+	 * @song_info: the #RBSongInfo instance
+	 * @entry: the #RhythmDBEntry being changed
+	 *
+	 * Emitted just before the changes made in the song properties window
+	 * are applied to the database.  This is only emitted in the single-entry
+	 * case.
+	 */
 	rb_song_info_signals[PRE_METADATA_CHANGE] =
 		g_signal_new ("pre-metadata-change",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -204,6 +294,14 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 			      1,
 			      RHYTHMDB_TYPE_ENTRY);
 
+	/**
+	 * RBSongInfo::post-metadata-change:
+	 * @song_info: the #RBSongInfo instance
+	 * @entry: the #RhythmDBEntry that was changed
+	 *
+	 * Emitted just after changes have been applied to the database.
+	 * Probably useless.
+	 */
 	rb_song_info_signals[POST_METADATA_CHANGE] =
 		g_signal_new ("post-metadata-change",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -277,6 +375,7 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	song_info->priv->duration      = glade_xml_get_widget (xml, "song_info_duration");
 	song_info->priv->location = glade_xml_get_widget (xml, "song_info_location");
 	song_info->priv->filesize = glade_xml_get_widget (xml, "song_info_filesize");
+	song_info->priv->date_added    = glade_xml_get_widget (xml, "song_info_dateadded");
 	song_info->priv->play_count    = glade_xml_get_widget (xml, "song_info_playcount");
 	song_info->priv->last_played   = glade_xml_get_widget (xml, "song_info_lastplayed");
 	song_info->priv->name = glade_xml_get_widget (xml, "song_info_name");
@@ -286,6 +385,7 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	rb_glade_boldify_label (xml, "name_label");
 	rb_glade_boldify_label (xml, "location_label");
 	rb_glade_boldify_label (xml, "filesize_label");
+	rb_glade_boldify_label (xml, "date_added_label");
 	rb_glade_boldify_label (xml, "last_played_label");
 	rb_glade_boldify_label (xml, "play_count_label");
 	rb_glade_boldify_label (xml, "duration_label");
@@ -468,7 +568,7 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 }
 
 static void
-rb_song_info_finalize (GObject *object)
+rb_song_info_dispose (GObject *object)
 {
 	RBSongInfo *song_info;
 
@@ -479,17 +579,49 @@ rb_song_info_finalize (GObject *object)
 
 	g_return_if_fail (song_info->priv != NULL);
 
-	g_signal_handlers_disconnect_by_func (song_info->priv->source,
-					      G_CALLBACK (rb_song_info_query_model_changed_cb),
-					      song_info);
+	if (song_info->priv->albums != NULL) {
+		g_object_unref (song_info->priv->albums);
+		song_info->priv->albums = NULL;
+	}
+	if (song_info->priv->artists != NULL) {
+		g_object_unref (song_info->priv->artists);
+		song_info->priv->artists = NULL;
+	}
+	if (song_info->priv->genres != NULL) {
+		g_object_unref (song_info->priv->genres);
+		song_info->priv->genres = NULL;
+	}
 
-	g_object_unref (song_info->priv->albums);
-	g_object_unref (song_info->priv->artists);
-	g_object_unref (song_info->priv->genres);
+	if (song_info->priv->db != NULL) {
+		g_object_unref (song_info->priv->db);
+		song_info->priv->db = NULL;
+	}
+	if (song_info->priv->source != NULL) {
+		g_signal_handlers_disconnect_by_func (song_info->priv->source,
+						      G_CALLBACK (rb_song_info_query_model_changed_cb),
+						      song_info);
+		g_object_unref (song_info->priv->source);
+		song_info->priv->source = NULL;
+	}
+	if (song_info->priv->query_model != NULL) {
+		g_object_unref (song_info->priv->query_model);
+		song_info->priv->query_model = NULL;
+	}
 
-	g_object_unref (song_info->priv->db);
-	g_object_unref (song_info->priv->source);
-	g_object_unref (song_info->priv->query_model);
+	G_OBJECT_CLASS (rb_song_info_parent_class)->dispose (object);
+}
+
+static void
+rb_song_info_finalize (GObject *object)
+{
+	RBSongInfo *song_info;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_SONG_INFO (object));
+
+	song_info = RB_SONG_INFO (object);
+
+	g_return_if_fail (song_info->priv != NULL);
 
 	if (song_info->priv->selected_entries != NULL) {
 		g_list_foreach (song_info->priv->selected_entries, (GFunc)rhythmdb_entry_unref, NULL);
@@ -602,12 +734,40 @@ rb_song_info_get_property (GObject *object,
 	case PROP_CURRENT_ENTRY:
 		g_value_set_boxed (value, song_info->priv->current_entry);
 		break;
+	case PROP_SELECTED_ENTRIES:
+		if (song_info->priv->selected_entries) {
+			GValueArray *value_array;
+			GValue entry_value = { 0, };
+			GList *entry_list;
+
+			value_array = g_value_array_new (1);
+			g_value_init (&entry_value, RHYTHMDB_TYPE_ENTRY);
+			for (entry_list = song_info->priv->selected_entries; entry_list; entry_list = entry_list->next) {
+				g_value_set_boxed (&entry_value, entry_list->data);
+				g_value_array_append (value_array, &entry_value);
+			}
+			g_value_unset (&entry_value);
+			g_value_take_boxed (value, value_array);
+		} else {
+			g_value_set_boxed (value, NULL);
+		}
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
+/**
+ * rb_song_info_new:
+ * @source: #RBSource creating the song properties window
+ * @entry_view: the #RBEntryView to get selection data from
+ *
+ * Creates a new #RBSongInfo for the selected entry or entries in
+ * the specified entry view.
+ *
+ * Return value: the new song properties window
+ */
 GtkWidget *
 rb_song_info_new (RBSource *source, RBEntryView *entry_view)
 {
@@ -631,6 +791,17 @@ rb_song_info_new (RBSource *source, RBEntryView *entry_view)
 	return GTK_WIDGET (song_info);
 }
 
+/**
+ * rb_song_info_append_page:
+ * @info: a #RBSongInfo
+ * @title: the title of the new page
+ * @page: the page #GtkWidget
+ *
+ * Adds a new page to the song properties window.  Should be called
+ * in a handler connected to the #RBShell 'create-song-info' signal.
+ *
+ * Return value: the page number
+ */
 guint
 rb_song_info_append_page (RBSongInfo *info, const char *title, GtkWidget *page)
 {
@@ -852,6 +1023,7 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 	rb_song_info_update_duration (song_info);
 	rb_song_info_update_location (song_info);
 	rb_song_info_update_filesize (song_info);
+	rb_song_info_update_date_added (song_info);
 	rb_song_info_update_play_count (song_info);
 	rb_song_info_update_last_played (song_info);
 	rb_song_info_update_bitrate (song_info);
@@ -1169,6 +1341,14 @@ rb_song_info_update_year (RBSongInfo *song_info)
 	g_free (text);
 }
 
+static void
+rb_song_info_update_date_added (RBSongInfo *song_info)
+{
+	const char *str;
+	str = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_FIRST_SEEN_STR);
+	gtk_label_set_text (GTK_LABEL (song_info->priv->date_added), str);
+}
+
 static void
 rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 {
